@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004 Ian Berry                                            |
+ | Copyright (C) 2004-2008 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -13,14 +13,12 @@
  | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
  | GNU General Public License for more details.                            |
  +-------------------------------------------------------------------------+
- | cacti: a php-based graphing solution                                    |
+ | Cacti: The Complete RRDTool-based Graphing Solution                     |
  +-------------------------------------------------------------------------+
- | Most of this code has been designed, written and is maintained by       |
- | Ian Berry. See about.php for specific developer credit. Any questions   |
- | or comments regarding this code should be directed to:                  |
- | - iberry@raxnet.net                                                     |
+ | This code is designed, written, and maintained by the Cacti Group. See  |
+ | about.php and/or the AUTHORS file for specific developer information.   |
  +-------------------------------------------------------------------------+
- | - raXnet - http://www.raxnet.net/                                       |
+ | http://www.cacti.net/                                                   |
  +-------------------------------------------------------------------------+
 */
 
@@ -32,25 +30,25 @@ if (!isset($_REQUEST["action"])) { $_REQUEST["action"] = ""; }
 switch ($_REQUEST["action"]) {
 	case 'save':
 		form_save();
-		
-		break;          
+
+		break;
 	case 'remove':
 		color_remove();
-	    
+
 		header ("Location: color.php");
 		break;
 	case 'edit':
 		include_once("./include/top_header.php");
-		
+
 		color_edit();
-		
+
 		include_once("./include/bottom_footer.php");
 		break;
 	default:
 		include_once("./include/top_header.php");
-		
+
 		color();
-		
+
 		include_once("./include/bottom_footer.php");
 		break;
 }
@@ -63,17 +61,17 @@ function form_save() {
 	if (isset($_POST["save_component_color"])) {
 		$save["id"] = $_POST["id"];
 		$save["hex"] = form_input_validate($_POST["hex"], "hex", "^[a-fA-F0-9]+$", false, 3);
-		
+
 		if (!is_error_message()) {
 			$color_id = sql_save($save, "colors");
-			
+
 			if ($color_id) {
 				raise_message(1);
 			}else{
 				raise_message(2);
 			}
 		}
-		
+
 		if (is_error_message()) {
 			header("Location: color.php?action=edit&id=" . (empty($color_id) ? $_POST["id"] : $color_id));
 		}else{
@@ -87,36 +85,44 @@ function form_save() {
    ----------------------- */
 
 function color_remove() {
-	db_execute("delete from colors where id=" . $_GET["id"]);	
+	/* ================= input validation ================= */
+	input_validate_input_number(get_request_var("id"));
+	/* ==================================================== */
+
+	db_execute("delete from colors where id=" . $_GET["id"]);
 }
 
 function color_edit() {
 	global $colors, $fields_color_edit;
-	
+
+	/* ================= input validation ================= */
+	input_validate_input_number(get_request_var("id"));
+	/* ==================================================== */
+
 	if (!empty($_GET["id"])) {
 		$color = db_fetch_row("select * from colors where id=" . $_GET["id"]);
 		$header_label = "[edit: " . $color["hex"] . "]";
 	}else{
 		$header_label = "[new]";
 	}
-	
-	start_box("<strong>Colors</strong> $header_label", "98%", $colors["header"], "3", "center", "");
-    	
+
+	html_start_box("<strong>Colors</strong> $header_label", "100%", $colors["header"], "3", "center", "");
+
 	draw_edit_form(array(
 		"config" => array(),
 		"fields" => inject_form_variables($fields_color_edit, (isset($color) ? $color : array()))
 		));
-	
-	end_box();
-	
+
+	html_end_box();
+
 	form_save_button("color.php");
 }
 
 function color() {
 	global $colors;
-	
-	start_box("<strong>Colors</strong>", "98%", $colors["header"], "3", "center", "color.php?action=edit");
-	
+
+	html_start_box("<strong>Colors</strong>", "100%", $colors["header"], "3", "center", "color.php?action=edit");
+
 	print "<tr bgcolor='#" . $colors["header_panel"] . "'>";
 		DrawMatrixHeaderItem("Hex Value",$colors["header_text"],1);
 		DrawMatrixHeaderItem("Color",$colors["header_text"],1);
@@ -127,9 +133,9 @@ function color() {
                 DrawMatrixHeaderItem("Color",$colors["header_text"],1);
 		DrawMatrixHeaderItem("&nbsp;",$colors["header_text"],1);
 	print "</tr>";
-    
+
 	$color_list = db_fetch_assoc("select * from colors order by hex");
-	
+
 	$i = 0;
 	if (sizeof($color_list) > 0) {
 		$j=0; ## even/odd counter
@@ -142,8 +148,8 @@ function color() {
 						<a class="linkEditMain" href="color.php?action=edit&id=<?php print $color["id"];?>"><?php print $color["hex"];?></a>
 					</td>
 					<td bgcolor="#<?php print $color["hex"];?>" width="1%">&nbsp;</td>
-					<td width="1%" align="right">
-						<a href="color.php?action=remove&id=<?php print $color["id"];?>"><img src="images/delete_icon.gif" width="10" height="10" border="0" alt="Delete"></a>&nbsp;
+					<td align="right">
+						<a href="color.php?action=remove&id=<?php print $color["id"];?>"><img src="images/delete_icon.gif" width="10" height="10" border="0" alt="Delete"></a>
 					</td>
 				<?php	$j=1;
 			} else { ?>
@@ -152,8 +158,8 @@ function color() {
 						<a class="linkEditMain" href="color.php?action=edit&id=<?php print $color["id"];?>"><?php print $color["hex"];?></a>
 					</td>
 					<td bgcolor="#<?php print $color["hex"];?>" width="1%">&nbsp;</td>
-					<td width="1%" align="right">
-						<a href="color.php?action=remove&id=<?php print $color["id"];?>"><img src="images/delete_icon.gif" width="10" height="10" border="0" alt="Delete"></a>&nbsp;
+					<td align="right">
+						<a href="color.php?action=remove&id=<?php print $color["id"];?>"><img src="images/delete_icon.gif" width="10" height="10" border="0" alt="Delete"></a>
 					</td>
 				</tr>
 			<?php
@@ -167,7 +173,7 @@ function color() {
 			<?php
 		}
 	}
-	end_box();	
+	html_end_box();
 }
-   
+
 ?>
