@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004 Ian Berry                                            |
+ | Copyright (C) 2004-2007 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -13,14 +13,12 @@
  | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
  | GNU General Public License for more details.                            |
  +-------------------------------------------------------------------------+
- | cacti: a php-based graphing solution                                    |
+ | Cacti: The Complete RRDTool-based Graphing Solution                     |
  +-------------------------------------------------------------------------+
- | Most of this code has been designed, written and is maintained by       |
- | Ian Berry. See about.php for specific developer credit. Any questions   |
- | or comments regarding this code should be directed to:                  |
- | - iberry@raxnet.net                                                     |
+ | This code is designed, written, and maintained by the Cacti Group. See  |
+ | about.php and/or the AUTHORS file for specific developer information.   |
  +-------------------------------------------------------------------------+
- | - raXnet - http://www.raxnet.net/                                       |
+ | http://www.cacti.net/                                                   |
  +-------------------------------------------------------------------------+
 */
 
@@ -32,28 +30,50 @@ $guest_account = true;
 include("./include/auth.php");
 include_once("./lib/rrd.php");
 
+/* ================= input validation ================= */
+input_validate_input_number(get_request_var("graph_start"));
+input_validate_input_number(get_request_var("graph_end"));
+input_validate_input_number(get_request_var("graph_height"));
+input_validate_input_number(get_request_var("graph_width"));
+input_validate_input_number(get_request_var("local_graph_id"));
+input_validate_input_number(get_request_var("rra_id"));
+/* ==================================================== */
+
 header("Content-type: image/png");
 
 /* flush the headers now */
-ob_end_flush();
+ob_end_clean();
 
 session_write_close();
 
 $graph_data_array = array();
 
-/* check to see if there are user specified vars */
-if (!empty($_GET["graph_start"])) {
-	$graph_data_array["use"] = true;
+/* override: graph start time (unix time) */
+if (!empty($_GET["graph_start"]) && $_GET["graph_start"] < 1600000000) {
 	$graph_data_array["graph_start"] = $_GET["graph_start"];
+}
+
+/* override: graph end time (unix time) */
+if (!empty($_GET["graph_end"]) && $_GET["graph_end"] < 1600000000) {
+	$graph_data_array["graph_end"] = $_GET["graph_end"];
+}
+
+/* override: graph height (in pixels) */
+if (!empty($_GET["graph_height"]) && $_GET["graph_height"] < 3000) {
 	$graph_data_array["graph_height"] = $_GET["graph_height"];
+}
+
+/* override: graph width (in pixels) */
+if (!empty($_GET["graph_width"]) && $_GET["graph_width"] < 3000) {
 	$graph_data_array["graph_width"] = $_GET["graph_width"];
 }
 
-/* treat the legend separatly */
+/* override: skip drawing the legend? */
 if (!empty($_GET["graph_nolegend"])) {
 	$graph_data_array["graph_nolegend"] = $_GET["graph_nolegend"];
 }
 
+/* print RRDTool graph source? */
 if (!empty($_GET["show_source"])) {
 	$graph_data_array["print_source"] = $_GET["show_source"];
 }
