@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004 Ian Berry                                            |
+ | Copyright (C) 2004-2007 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -13,14 +13,12 @@
  | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
  | GNU General Public License for more details.                            |
  +-------------------------------------------------------------------------+
- | cacti: a php-based graphing solution                                    |
+ | Cacti: The Complete RRDTool-based Graphing Solution                     |
  +-------------------------------------------------------------------------+
- | Most of this code has been designed, written and is maintained by       |
- | Ian Berry. See about.php for specific developer credit. Any questions   |
- | or comments regarding this code should be directed to:                  |
- | - iberry@raxnet.net                                                     |
+ | This code is designed, written, and maintained by the Cacti Group. See  |
+ | about.php and/or the AUTHORS file for specific developer information.   |
  +-------------------------------------------------------------------------+
- | - raXnet - http://www.raxnet.net/                                       |
+ | http://www.cacti.net/                                                   |
  +-------------------------------------------------------------------------+
 */
 
@@ -41,6 +39,8 @@ $graph_actions = array(
 	1 => "Delete",
 	2 => "Change Graph Template",
 	5 => "Change Host",
+	6 => "Reapply Suggested Names",
+	7 => "Resize Graphs",
 	3 => "Duplicate",
 	4 => "Convert to Graph Template"
 	);
@@ -115,6 +115,10 @@ function add_tree_names_to_actions_array() {
 
 function form_save() {
 	if ((isset($_POST["save_component_graph_new"])) && (!empty($_POST["graph_template_id"]))) {
+		/* ================= input validation ================= */
+		input_validate_input_number(get_request_var_post("graph_template_id"));
+		/* ==================================================== */
+
 		$save["id"] = $_POST["local_graph_id"];
 		$save["graph_template_id"] = $_POST["graph_template_id"];
 		$save["host_id"] = $_POST["host_id"];
@@ -128,6 +132,11 @@ function form_save() {
 	}
 
 	if (isset($_POST["save_component_graph"])) {
+		/* ================= input validation ================= */
+		input_validate_input_number(get_request_var_post("graph_template_id"));
+		input_validate_input_number(get_request_var_post("_graph_template_id"));
+		/* ==================================================== */
+
 		$save1["id"] = $_POST["local_graph_id"];
 		$save1["host_id"] = $_POST["host_id"];
 		$save1["graph_template_id"] = $_POST["graph_template_id"];
@@ -139,12 +148,14 @@ function form_save() {
 		$save2["title"] = form_input_validate($_POST["title"], "title", "", false, 3);
 		$save2["height"] = form_input_validate($_POST["height"], "height", "^[0-9]+$", false, 3);
 		$save2["width"] = form_input_validate($_POST["width"], "width", "^[0-9]+$", false, 3);
-		$save2["upper_limit"] = form_input_validate($_POST["upper_limit"], "upper_limit", "^-?[0-9]+$", false, 3);
-		$save2["lower_limit"] = form_input_validate($_POST["lower_limit"], "lower_limit", "^-?[0-9]+$", false, 3);
+		$save2["upper_limit"] = form_input_validate($_POST["upper_limit"], "upper_limit", "^(-?([0-9]+(\.[0-9]*)?|[0-9]*\.[0-9]+)([eE][+\-]?[0-9]+)?)|U$", false, 3);
+		$save2["lower_limit"] = form_input_validate($_POST["lower_limit"], "lower_limit", "^(-?([0-9]+(\.[0-9]*)?|[0-9]*\.[0-9]+)([eE][+\-]?[0-9]+)?)|U$", false, 3);
 		$save2["vertical_label"] = form_input_validate($_POST["vertical_label"], "vertical_label", "", true, 3);
+		$save2["slope_mode"] = form_input_validate((isset($_POST["slope_mode"]) ? $_POST["slope_mode"] : ""), "slope_mode", "", true, 3);
 		$save2["auto_scale"] = form_input_validate((isset($_POST["auto_scale"]) ? $_POST["auto_scale"] : ""), "auto_scale", "", true, 3);
 		$save2["auto_scale_opts"] = form_input_validate($_POST["auto_scale_opts"], "auto_scale_opts", "", true, 3);
 		$save2["auto_scale_log"] = form_input_validate((isset($_POST["auto_scale_log"]) ? $_POST["auto_scale_log"] : ""), "auto_scale_log", "", true, 3);
+		$save2["scale_log_units"] = form_input_validate((isset($_POST["scale_log_units"]) ? $_POST["scale_log_units"] : ""), "scale_log_units", "", true, 3);
 		$save2["auto_scale_rigid"] = form_input_validate((isset($_POST["auto_scale_rigid"]) ? $_POST["auto_scale_rigid"] : ""), "auto_scale_rigid", "", true, 3);
 		$save2["auto_padding"] = form_input_validate((isset($_POST["auto_padding"]) ? $_POST["auto_padding"] : ""), "auto_padding", "", true, 3);
 		$save2["base_value"] = form_input_validate($_POST["base_value"], "base_value", "^[0-9]+$", false, 3);
@@ -192,6 +203,10 @@ function form_save() {
 	}
 
 	if (isset($_POST["save_component_input"])) {
+		/* ================= input validation ================= */
+		input_validate_input_number(get_request_var_post("local_graph_id"));
+		/* ==================================================== */
+
 		/* first; get the current graph template id */
 		$graph_template_id = db_fetch_cell("select graph_template_id from graph_local where id=" . $_POST["local_graph_id"]);
 
@@ -203,7 +218,7 @@ function form_save() {
 			/* we need to find out which graph items will be affected by saving this particular item */
 			$item_list = db_fetch_assoc("select
 				graph_templates_item.id
-				from graph_template_input_defs,graph_templates_item
+				from (graph_template_input_defs,graph_templates_item)
 				where graph_template_input_defs.graph_template_item_id=graph_templates_item.local_graph_template_item_id
 				and graph_templates_item.local_graph_id=" . $_POST["local_graph_id"] . "
 				and graph_template_input_defs.graph_template_input_id=" . $input["id"]);
@@ -248,20 +263,23 @@ function form_save() {
 
 function form_actions() {
 	global $colors, $graph_actions;
-
 	/* if we are to save this form, instead of display it */
 	if (isset($_POST["selected_items"])) {
 		$selected_items = unserialize(stripslashes($_POST["selected_items"]));
 
 		if ($_POST["drp_action"] == "1") { /* delete */
 			for ($i=0;($i<count($selected_items));$i++) {
+				/* ================= input validation ================= */
+				input_validate_input_number($selected_items[$i]);
+				/* ==================================================== */
+
 				if (!isset($_POST["delete_type"])) { $_POST["delete_type"] = 1; }
 
 				switch ($_POST["delete_type"]) {
 					case '2': /* delete all data sources referenced by this graph */
 						$data_sources = db_fetch_assoc("select
 							data_template_data.local_data_id
-							from data_template_rrd,data_template_data,graph_templates_item
+							from (data_template_rrd,data_template_data,graph_templates_item)
 							where graph_templates_item.task_item_id=data_template_rrd.id
 							and data_template_rrd.local_data_id=data_template_data.local_data_id
 							and " . array_to_sql_or($selected_items, "graph_templates_item.local_graph_id") . "
@@ -281,25 +299,66 @@ function form_actions() {
 			}
 		}elseif ($_POST["drp_action"] == "2") { /* change graph template */
 			for ($i=0;($i<count($selected_items));$i++) {
+				/* ================= input validation ================= */
+				input_validate_input_number($selected_items[$i]);
+				input_validate_input_number(get_request_var_post("graph_template_id"));
+				/* ==================================================== */
+
 				change_graph_template($selected_items[$i], $_POST["graph_template_id"], true);
 			}
 		}elseif ($_POST["drp_action"] == "3") { /* duplicate */
 			for ($i=0;($i<count($selected_items));$i++) {
+				/* ================= input validation ================= */
+				input_validate_input_number($selected_items[$i]);
+				/* ==================================================== */
+
 				duplicate_graph($selected_items[$i], 0, $_POST["title_format"]);
 			}
 		}elseif ($_POST["drp_action"] == "4") { /* graph -> graph template */
 			for ($i=0;($i<count($selected_items));$i++) {
+				/* ================= input validation ================= */
+				input_validate_input_number($selected_items[$i]);
+				/* ==================================================== */
+
 				graph_to_graph_template($selected_items[$i], $_POST["title_format"]);
 			}
 		}elseif (ereg("^tr_([0-9]+)$", $_POST["drp_action"], $matches)) { /* place on tree */
 			for ($i=0;($i<count($selected_items));$i++) {
+				/* ================= input validation ================= */
+				input_validate_input_number($selected_items[$i]);
+				input_validate_input_number(get_request_var_post("tree_id"));
+				input_validate_input_number(get_request_var_post("tree_item_id"));
+				/* ==================================================== */
+
 				api_tree_item_save(0, $_POST["tree_id"], TREE_ITEM_TYPE_GRAPH, $_POST["tree_item_id"], "", $selected_items[$i], read_graph_config_option("default_rra_id"), 0, 0, 0, false);
 			}
 		}elseif ($_POST["drp_action"] == "5") { /* change host */
 			for ($i=0;($i<count($selected_items));$i++) {
+				/* ================= input validation ================= */
+				input_validate_input_number($selected_items[$i]);
+				input_validate_input_number(get_request_var_post("host_id"));
+				/* ==================================================== */
+
 				db_execute("update graph_local set host_id=" . $_POST["host_id"] . " where id=" . $selected_items[$i]);
 				update_graph_title_cache($selected_items[$i]);
 			}
+		}elseif ($_POST["drp_action"] == "6") { /* reapply suggested naming */
+			for ($i=0;($i<count($selected_items));$i++) {
+				/* ================= input validation ================= */
+				input_validate_input_number($selected_items[$i]);
+				/* ==================================================== */
+
+				api_reapply_suggested_graph_title($selected_items[$i]);
+				update_graph_title_cache($selected_items[$i]);
+			}
+		}elseif ($_POST["drp_action"] == "7") { /* resize graphs */
+			for ($i=0;($i<count($selected_items));$i++) {
+				/* ================= input validation ================= */
+				input_validate_input_number($selected_items[$i]);
+				/* ==================================================== */
+
+				api_resize_graphs($selected_items[$i], $_POST["graph_width"], $_POST["graph_height"]);
+			}
 		}
 
 		header("Location: graphs.php");
@@ -312,6 +371,10 @@ function form_actions() {
 	/* loop through each of the graphs selected on the previous page and get more info about them */
 	while (list($var,$val) = each($_POST)) {
 		if (ereg("^chk_([0-9]+)$", $var, $matches)) {
+			/* ================= input validation ================= */
+			input_validate_input_number($matches[1]);
+			/* ==================================================== */
+
 			$graph_list .= "<li>" . get_graph_title($matches[1]) . "<br>";
 			$graph_array[$i] = $matches[1];
 		}
@@ -336,7 +399,7 @@ function form_actions() {
 			$data_sources = db_fetch_assoc("select
 				data_template_data.local_data_id,
 				data_template_data.name_cache
-				from data_template_rrd,data_template_data,graph_templates_item
+				from (data_template_rrd,data_template_data,graph_templates_item)
 				where graph_templates_item.task_item_id=data_template_rrd.id
 				and data_template_rrd.local_data_id=data_template_data.local_data_id
 				and " . array_to_sql_or($graph_array, "graph_templates_item.local_graph_id") . "
@@ -373,7 +436,7 @@ function form_actions() {
 					the following graphs. Be aware that all warnings will be suppressed during the
 					conversion, so graph data loss is possible.</p>
 					<p>$graph_list</p>
-					<p><strong>New Graph Template:</strong><br>"; form_dropdown("graph_template_id",db_fetch_assoc("select graph_templates.id,graph_templates.name from graph_templates"),"name","id","","","0"); print "</p>
+					<p><strong>New Graph Template:</strong><br>"; form_dropdown("graph_template_id",db_fetch_assoc("select graph_templates.id,graph_templates.name from graph_templates order by name"),"name","id","","","0"); print "</p>
 				</td>
 			</tr>\n
 			";
@@ -417,6 +480,25 @@ function form_actions() {
 				</td>
 			</tr>\n
 			";
+	}elseif ($_POST["drp_action"] == "6") { /* reapply suggested naming to host */
+		print "	<tr>
+				<td class='textArea' bgcolor='#" . $colors["form_alternate1"]. "'>
+					<p>When you click save, the following graphs will have thier suggested naming convensions
+					recalculated and applies to the graphs.</p>
+					<p>$graph_list</p>
+				</td>
+			</tr>\n
+			";
+	}elseif ($_POST["drp_action"] == "7") { /* reapply suggested naming to host */
+		print "	<tr>
+				<td class='textArea' bgcolor='#" . $colors["form_alternate1"]. "'>
+					<p>When you click save, the following graphs will be resized per your specifications.</p>
+					<p>$graph_list</p>
+					<p><strong>Graph Height:</strong><br>"; form_text_box("graph_height", "", "", "255", "30", "text"); print "</p>
+					<p><strong>Graph Width:</strong><br>"; form_text_box("graph_width", "", "", "255", "30", "text"); print "</p>
+				</td>
+			</tr>\n
+			";
 	}
 
 	if (!isset($graph_array)) {
@@ -449,6 +531,10 @@ function form_actions() {
 function item() {
 	global $colors, $consolidation_functions, $graph_item_types, $struct_graph_item;
 
+	/* ================= input validation ================= */
+	input_validate_input_number(get_request_var("id"));
+	/* ==================================================== */
+
 	if (empty($_GET["id"])) {
 		$template_item_list = array();
 
@@ -465,11 +551,11 @@ function item() {
 			cdef.name as cdef_name,
 			colors.hex
 			from graph_templates_item
-			left join data_template_rrd on graph_templates_item.task_item_id=data_template_rrd.id
-			left join data_local on data_template_rrd.local_data_id=data_local.id
-			left join data_template_data on data_local.id=data_template_data.local_data_id
-			left join cdef on cdef_id=cdef.id
-			left join colors on color_id=colors.id
+			left join data_template_rrd on (graph_templates_item.task_item_id=data_template_rrd.id)
+			left join data_local on (data_template_rrd.local_data_id=data_local.id)
+			left join data_template_data on (data_local.id=data_template_data.local_data_id)
+			left join cdef on (cdef_id=cdef.id)
+			left join colors on (color_id=colors.id)
 			where graph_templates_item.local_graph_id=" . $_GET["id"] . "
 			order by graph_templates_item.sequence");
 
@@ -485,7 +571,7 @@ function item() {
 		$add_text = "";
 	}
 
-	html_start_box("<strong>Graph Items</strong> $header_label", "98%", $colors["header"], "3", "center", $add_text);
+	html_start_box("<strong>Graph Items</strong> $header_label", "100%", $colors["header"], "3", "center", $add_text);
 	draw_graph_items_list($template_item_list, "graphs_items.php", "local_graph_id=" . $_GET["id"], (empty($graph_template_id) ? false : true));
 	html_end_box();
 }
@@ -497,6 +583,11 @@ function item() {
 function graph_diff() {
 	global $colors, $struct_graph_item, $graph_item_types, $consolidation_functions;
 
+	/* ================= input validation ================= */
+	input_validate_input_number(get_request_var("id"));
+	input_validate_input_number(get_request_var("graph_template_id"));
+	/* ==================================================== */
+
 	$template_query = "select
 		graph_templates_item.id,
 		graph_templates_item.text_format,
@@ -507,11 +598,12 @@ function graph_diff() {
 		CONCAT_WS(' - ',data_template_data.name,data_template_rrd.data_source_name) as task_item_id,
 		cdef.name as cdef_id,
 		colors.hex as color_id
-		from graph_templates_item left join data_template_rrd on graph_templates_item.task_item_id=data_template_rrd.id
-		left join data_local on data_template_rrd.local_data_id=data_local.id
-		left join data_template_data on data_local.id=data_template_data.local_data_id
-		left join cdef on cdef_id=cdef.id
-		left join colors on color_id=colors.id";
+		from graph_templates_item
+		left join data_template_rrd on (graph_templates_item.task_item_id=data_template_rrd.id)
+		left join data_local on (data_template_rrd.local_data_id=data_local.id)
+		left join data_template_data on (data_local.id=data_template_data.local_data_id)
+		left join cdef on (cdef_id=cdef.id)
+		left join colors on (color_id=colors.id)";
 
 	/* first, get information about the graph template as that's what we're going to model this
 	graph after */
@@ -530,7 +622,7 @@ function graph_diff() {
 	$graph_template_inputs = db_fetch_assoc("select
 		graph_template_input.column_name,
 		graph_template_input_defs.graph_template_item_id
-		from graph_template_input,graph_template_input_defs
+		from (graph_template_input,graph_template_input_defs)
 		where graph_template_input.id=graph_template_input_defs.graph_template_input_id
 		and graph_template_input.graph_template_id=" . $_GET["graph_template_id"]);
 
@@ -543,7 +635,7 @@ function graph_diff() {
 	}
 
 	?>
-	<table style="background-color: #f5f5f5; border: 1px solid #aaaaaa;" width="98%" align="center">
+	<table style="background-color: #f5f5f5; border: 1px solid #aaaaaa;" width="100%" align="center">
 		<tr>
 			<td class="textArea">
 				The template you have selected requires some changes to be made to the structure of
@@ -555,7 +647,7 @@ function graph_diff() {
 	<br>
 	<?php
 
-	html_start_box("<strong>Graph Preview</strong>", "98%", $colors["header"], "3", "center", "");
+	html_start_box("<strong>Graph Preview</strong>", "100%", $colors["header"], "3", "center", "");
 
 	$graph_item_actions = array("normal" => "", "add" => "+", "delete" => "-");
 
@@ -695,7 +787,7 @@ function graph_diff() {
 
 	?>
 	<form action="graphs.php" method="post">
-	<table style="background-color: #f5f5f5; border: 1px solid #aaaaaa;" width="98%" align="center">
+	<table style="background-color: #f5f5f5; border: 1px solid #aaaaaa;" width="100%" align="center">
 		<tr>
 			<td class="textArea">
 				<input type='radio' name='type' value='1' checked>&nbsp;<?php print $user_message;?><br>
@@ -718,6 +810,10 @@ function graph_diff() {
 function graph_edit() {
 	global $colors, $struct_graph, $image_types, $consolidation_functions, $graph_item_types, $struct_graph_item;
 
+	/* ================= input validation ================= */
+	input_validate_input_number(get_request_var("id"));
+	/* ==================================================== */
+
 	$use_graph_template = true;
 
 	if (!empty($_GET["id"])) {
@@ -748,7 +844,7 @@ function graph_edit() {
 
 	if (!empty($_GET["id"])) {
 		?>
-		<table width="98%" align="center">
+		<table width="100%" align="center">
 			<tr>
 				<td class="textInfo" colspan="2" valign="top">
 					<?php print get_graph_title($_GET["id"]);?>
@@ -762,7 +858,7 @@ function graph_edit() {
 		<?php
 	}
 
-	html_start_box("<strong>Graph Template Selection</strong> $header_label", "98%", $colors["header"], "3", "center", "");
+	html_start_box("<strong>Graph Template Selection</strong> $header_label", "100%", $colors["header"], "3", "center", "");
 
 	$form_array = array(
 		"graph_template_id" => array(
@@ -814,7 +910,7 @@ function graph_edit() {
 
 	/* only display the "inputs" area if we are using a graph template for this graph */
 	if (!empty($graphs["graph_template_id"])) {
-		html_start_box("<strong>Supplemental Graph Template Data</strong>", "98%", $colors["header"], "3", "center", "");
+		html_start_box("<strong>Supplemental Graph Template Data</strong>", "100%", $colors["header"], "3", "center", "");
 
 		print "<form method='post' action='graphs.php'>\n";
 
@@ -831,17 +927,21 @@ function graph_edit() {
 
 	if (!empty($_GET["id"])) {
 		?>
-		<table width="98%" align="center">
+		<table width="100%" align="center">
 			<tr>
 				<td align="center" class="textInfo" colspan="2">
-					<img src="graph_image.php?local_graph_id=<?php print $_GET["id"];?>&rra_id=1" alt="">
+					<img src="graph_image.php?action=edit&local_graph_id=<?php print $_GET["id"];?>&rra_id=<?php print read_graph_config_option("default_rra_id");?>" alt="">
 				</td>
 				<?php
 				if ((isset($_SESSION["graph_debug_mode"])) && (isset($_GET["id"]))) {
 					$graph_data_array["output_flag"] = RRDTOOL_OUTPUT_STDERR;
+					$graph_data_array["print_source"] = 1;
 					?>
 					<td>
+						<span class="textInfo">RRDTool Command:</span><br>
+						<pre><?php print rrdtool_function_graph($_GET["id"], 1, $graph_data_array);?></pre>
 						<span class="textInfo">RRDTool Says:</span><br>
+						<?php unset($graph_data_array["print_source"]);?>
 						<pre><?php print rrdtool_function_graph($_GET["id"], 1, $graph_data_array);?></pre>
 					</td>
 					<?php
@@ -854,7 +954,7 @@ function graph_edit() {
 	}
 
 	if (((isset($_GET["id"])) || (isset($_GET["new"]))) && (empty($graphs["graph_template_id"]))) {
-		html_start_box("<strong>Graph Configuration</strong>", "98%", $colors["header"], "3", "center", "");
+		html_start_box("<strong>Graph Configuration</strong>", "100%", $colors["header"], "3", "center", "");
 
 		$form_array = array();
 
@@ -889,54 +989,142 @@ function graph_edit() {
 		form_hidden_box("save_component_graph_new","1","");
 	}
 
+	form_hidden_box("rrdtool_version", read_config_option("rrdtool_version"), "");
 	form_save_button("graphs.php");
+
+//Now we need some javascript to make it dynamic
+?>
+<script language="JavaScript">
+
+dynamic();
+
+function dynamic() {
+	//alert("RRDTool Version is '" + document.getElementById('rrdtool_version').value + "'");
+	//alert("Log is '" + document.getElementById('auto_scale_log').checked + "'");
+	document.getElementById('scale_log_units').disabled=true;
+	if ((document.getElementById('rrdtool_version').value != 'rrd-1.0.x') &&
+		(document.getElementById('auto_scale_log').checked)) {
+		document.getElementById('scale_log_units').disabled=false;
+	}
+}
+
+function changeScaleLog() {
+	//alert("Log changed to '" + document.getElementById('auto_scale_log').checked + "'");
+	document.getElementById('scale_log_units').disabled=true;
+	if ((document.getElementById('rrdtool_version').value != 'rrd-1.0.x') &&
+		(document.getElementById('auto_scale_log').checked)) {
+		document.getElementById('scale_log_units').disabled=false;
+	}
+}
+</script>
+<?php
+
 }
 
 function graph() {
 	global $colors, $graph_actions;
 
+	/* ================= input validation ================= */
+	input_validate_input_number(get_request_var_request("host_id"));
+	input_validate_input_number(get_request_var_request("template_id"));
+	input_validate_input_number(get_request_var_request("page"));
+	/* ==================================================== */
+
+	/* clean up search string */
+	if (isset($_REQUEST["filter"])) {
+		$_REQUEST["filter"] = sanitize_search_string(get_request_var("filter"));
+	}
+
+	/* clean up sort_column string */
+	if (isset($_REQUEST["sort_column"])) {
+		$_REQUEST["sort_column"] = sanitize_search_string(get_request_var("sort_column"));
+	}
+
+	/* clean up sort_direction string */
+	if (isset($_REQUEST["sort_direction"])) {
+		$_REQUEST["sort_direction"] = sanitize_search_string(get_request_var("sort_direction"));
+	}
+
 	/* if the user pushed the 'clear' button */
 	if (isset($_REQUEST["clear_x"])) {
 		kill_session_var("sess_graph_current_page");
 		kill_session_var("sess_graph_filter");
+		kill_session_var("sess_graph_sort_column");
+		kill_session_var("sess_graph_sort_direction");
 		kill_session_var("sess_graph_host_id");
+		kill_session_var("sess_graph_template_id");
 
 		unset($_REQUEST["page"]);
 		unset($_REQUEST["filter"]);
+		unset($_REQUEST["sort_column"]);
+		unset($_REQUEST["sort_direction"]);
 		unset($_REQUEST["host_id"]);
+		unset($_REQUEST["template_id"]);
 	}
 
 	/* remember these search fields in session vars so we don't have to keep passing them around */
 	load_current_session_value("page", "sess_graph_current_page", "1");
 	load_current_session_value("filter", "sess_graph_filter", "");
+	load_current_session_value("sort_column", "sess_graph_sort_column", "title_cache");
+	load_current_session_value("sort_direction", "sess_graph_sort_direction", "ASC");
 	load_current_session_value("host_id", "sess_graph_host_id", "-1");
+	load_current_session_value("template_id", "sess_graph_template_id", "-1");
 
-	html_start_box("<strong>Graph Management</strong>", "98%", $colors["header"], "3", "center", "graphs.php?action=graph_edit&host_id=" . $_REQUEST["host_id"]);
+	?>
+	<script type="text/javascript">
+	<!--
+
+	function applyGraphsFilterChange(objForm) {
+		strURL = '?host_id=' + objForm.host_id.value;
+		strURL = strURL + '&filter=' + objForm.filter.value;
+		strURL = strURL + '&template_id=' + objForm.template_id.value;
+		document.location = strURL;
+	}
+
+	-->
+	</script>
+	<?php
+
+	html_start_box("<strong>Graph Management</strong>", "100%", $colors["header"], "3", "center", "graphs.php?action=graph_edit&host_id=" . $_REQUEST["host_id"]);
 
 	include("./include/html/inc_graph_filter_table.php");
 
 	html_end_box();
 
 	/* form the 'where' clause for our main sql query */
-	$sql_where = "and graph_templates_graph.title_cache like '%%" . $_REQUEST["filter"] . "%%'";
+	if (strlen($_REQUEST["filter"])) {
+		$sql_where = "AND (graph_templates_graph.title_cache like '%%" . $_REQUEST["filter"] . "%%'" .
+			" OR graph_templates.name like '%%" . $_REQUEST["filter"] . "%%')";
+	}else{
+		$sql_where = "";
+	}
 
 	if ($_REQUEST["host_id"] == "-1") {
 		/* Show all items */
 	}elseif ($_REQUEST["host_id"] == "0") {
-		$sql_where .= " and graph_local.host_id=0";
+		$sql_where .= " AND graph_local.host_id=0";
 	}elseif (!empty($_REQUEST["host_id"])) {
-		$sql_where .= " and graph_local.host_id=" . $_REQUEST["host_id"];
+		$sql_where .= " AND graph_local.host_id=" . $_REQUEST["host_id"];
 	}
 
-	html_start_box("", "98%", $colors["header"], "3", "center", "");
+	if ($_REQUEST["template_id"] == "-1") {
+		/* Show all items */
+	}elseif ($_REQUEST["template_id"] == "0") {
+		$sql_where .= " AND graph_templates_graph.graph_template_id=0";
+	}elseif (!empty($_REQUEST["template_id"])) {
+		$sql_where .= " AND graph_templates_graph.graph_template_id=" . $_REQUEST["template_id"];
+	}
 
-	$total_rows = db_fetch_cell("select
+	html_start_box("", "100%", $colors["header"], "3", "center", "");
+
+	$total_rows = db_fetch_cell("SELECT
 		COUNT(graph_templates_graph.id)
-		from graph_local,graph_templates_graph
-		where graph_local.id=graph_templates_graph.local_graph_id
+		FROM (graph_local,graph_templates_graph)
+		LEFT JOIN graph_templates ON (graph_local.graph_template_id=graph_templates.id)
+		WHERE graph_local.id=graph_templates_graph.local_graph_id
 		$sql_where");
 
-	$graph_list = db_fetch_assoc("select
+	$graph_list = db_fetch_assoc("SELECT
 		graph_templates_graph.id,
 		graph_templates_graph.local_graph_id,
 		graph_templates_graph.height,
@@ -944,12 +1132,12 @@ function graph() {
 		graph_templates_graph.title_cache,
 		graph_templates.name,
 		graph_local.host_id
-		from graph_local,graph_templates_graph
-		left join graph_templates on graph_local.graph_template_id=graph_templates.id
-		where graph_local.id=graph_templates_graph.local_graph_id
+		FROM (graph_local,graph_templates_graph)
+		LEFT JOIN graph_templates ON (graph_local.graph_template_id=graph_templates.id)
+		WHERE graph_local.id=graph_templates_graph.local_graph_id
 		$sql_where
-		order by graph_templates_graph.title_cache,graph_local.host_id
-		limit " . (read_config_option("num_rows_graph")*($_REQUEST["page"]-1)) . "," . read_config_option("num_rows_graph"));
+		ORDER BY " . $_REQUEST['sort_column'] . " " . $_REQUEST['sort_direction'] .
+		" LIMIT " . (read_config_option("num_rows_graph")*($_REQUEST["page"]-1)) . "," . read_config_option("num_rows_graph"));
 
 	/* generate page list */
 	$url_page_select = get_page_list($_REQUEST["page"], MAX_DISPLAY_PAGES, read_config_option("num_rows_graph"), $total_rows, "graphs.php?filter=" . $_REQUEST["filter"] . "&host_id=" . $_REQUEST["host_id"]);
@@ -974,27 +1162,24 @@ function graph() {
 
 	print $nav;
 
-	html_header_checkbox(array("Graph Title", "Template Name", "Size"));
+	$display_text = array(
+		"title_cache" => array("Graph Title", "ASC"),
+		"name" => array("Template Name", "ASC"),
+		"height" => array("Size", "ASC"));
+
+	html_header_sort_checkbox($display_text, $_REQUEST["sort_column"], $_REQUEST["sort_direction"]);
 
 	$i = 0;
 	if (sizeof($graph_list) > 0) {
 		foreach ($graph_list as $graph) {
-			form_alternate_row_color($colors["alternate"],$colors["light"],$i); $i++;
-				?>
-				<td>
-					<a class="linkEditMain" href="graphs.php?action=graph_edit&id=<?php print $graph["local_graph_id"];?>"><?php print eregi_replace("(" . preg_quote($_REQUEST["filter"]) . ")", "<span style='background-color: #F8D93D;'>\\1</span>", title_trim($graph["title_cache"], read_config_option("max_title_graph")));?></a>
-				</td>
-				<td>
-					<?php print ((empty($graph["name"])) ? "<em>None</em>" : $graph["name"]); ?>
-				</td>
-				<td>
-					<?php print $graph["height"];?>x<?php print $graph["width"];?>
-				</td>
-				<td style="<?php print get_checkbox_style();?>" width="1%" align="right">
-					<input type='checkbox' style='margin: 0px;' name='chk_<?php print $graph["local_graph_id"];?>' title="<?php print $graph["title_cache"];?>">
-				</td>
-			</tr>
-			<?php
+			$template_name = ((empty($graph["name"])) ? "<em>None</em>" : $graph["name"]);
+
+			form_alternate_row_color($colors["alternate"], $colors["light"], $i, 'line' . $graph["local_graph_id"]); $i++;
+			form_selectable_cell("<a class='linkEditMain' href='graphs.php?action=graph_edit&id=" . $graph["local_graph_id"] . "' title='" . htmlspecialchars($graph["title_cache"]) . "'>" . (($_REQUEST["filter"] != "") ? eregi_replace("(" . preg_quote($_REQUEST["filter"]) . ")", "<span style='background-color: #F8D93D;'>\\1</span>", title_trim($graph["title_cache"], read_config_option("max_title_graph"))) : title_trim($graph["title_cache"], read_config_option("max_title_graph"))) . "</a>", $graph["local_graph_id"]);
+			form_selectable_cell((($_REQUEST["filter"] != "") ? eregi_replace("(" . preg_quote($_REQUEST["filter"]) . ")", "<span style='background-color: #F8D93D;'>\\1</span>", $template_name) : $template_name) . "</a>", $graph["local_graph_id"]);
+			form_selectable_cell($graph["height"] . "x" . $graph["width"], $graph["local_graph_id"]);
+			form_checkbox_cell($graph["title_cache"], $graph["local_graph_id"]);
+			form_end_row();
 		}
 
 		/* put the nav bar on the bottom as well */
@@ -1014,4 +1199,4 @@ function graph() {
 	print "</form>\n";
 }
 
-?>
\ No newline at end of file
+?>
