@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004 Ian Berry                                            |
+ | Copyright (C) 2004-2007 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -13,14 +13,12 @@
  | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
  | GNU General Public License for more details.                            |
  +-------------------------------------------------------------------------+
- | cacti: a php-based graphing solution                                    |
+ | Cacti: The Complete RRDTool-based Graphing Solution                     |
  +-------------------------------------------------------------------------+
- | Most of this code has been designed, written and is maintained by       |
- | Ian Berry. See about.php for specific developer credit. Any questions   |
- | or comments regarding this code should be directed to:                  |
- | - iberry@raxnet.net                                                     |
+ | This code is designed, written, and maintained by the Cacti Group. See  |
+ | about.php and/or the AUTHORS file for specific developer information.   |
  +-------------------------------------------------------------------------+
- | - raXnet - http://www.raxnet.net/                                       |
+ | http://www.cacti.net/                                                   |
  +-------------------------------------------------------------------------+
 */
 
@@ -33,28 +31,28 @@ if (!isset($_REQUEST["action"])) { $_REQUEST["action"] = ""; }
 switch ($_REQUEST["action"]) {
 	case 'save':
 		form_save();
-		
+
 		break;
 	case 'item_remove':
 		item_remove();
-		
+
 		header("Location: graphs.php?action=graph_edit&id=" . $_GET["local_graph_id"]);
 		break;
 	case 'item_edit':
 		include_once("./include/top_header.php");
-		
+
 		item_edit();
-		
+
 		include_once("./include/bottom_footer.php");
 		break;
 	case 'item_movedown':
 		item_movedown();
-		
+
 		header("Location: graphs.php?action=graph_edit&id=" . $_GET["local_graph_id"]);
 		break;
 	case 'item_moveup':
 		item_moveup();
-		
+
 		header("Location: graphs.php?action=graph_edit&id=" . $_GET["local_graph_id"]);
 		break;
 }
@@ -66,9 +64,9 @@ switch ($_REQUEST["action"]) {
 function form_save() {
 	if (isset($_POST["save_component_item"])) {
 		global $graph_item_types;
-		
+
 		$items[0] = array();
-		
+
 		if ($graph_item_types{$_POST["graph_type_id"]} == "LEGEND") {
 			/* this can be a major time saver when creating lots of graphs with the typical
 			GPRINT LAST/AVERAGE/MAX legends */
@@ -95,19 +93,21 @@ function form_save() {
 					"hard_return" => "on"
 					));
 		}
-		
-		foreach ($items as $item) {			
+
+		foreach ($items as $item) {
 			/* generate a new sequence if needed */
 			if (empty($_POST["sequence"])) {
 				$_POST["sequence"] = get_sequence($_POST["sequence"], "sequence", "graph_templates_item", "local_graph_id=" . $_POST["local_graph_id"]);
 			}
-			
 			$save["id"] = $_POST["graph_template_item_id"];
 			$save["graph_template_id"] = $_POST["graph_template_id"];
 			$save["local_graph_template_item_id"] = $_POST["local_graph_template_item_id"];
 			$save["local_graph_id"] = $_POST["local_graph_id"];
 			$save["task_item_id"] = form_input_validate($_POST["task_item_id"], "task_item_id", "", true, 3);
 			$save["color_id"] = form_input_validate((isset($item["color_id"]) ? $item["color_id"] : $_POST["color_id"]), "color_id", "", true, 3);
+			/* if alpha is disabled, use invisible_alpha instead */
+			if (!isset($_POST["alpha"])) {$_POST["alpha"] = $_POST["invisible_alpha"];}
+			$save["alpha"] = form_input_validate((isset($item["alpha"]) ? $item["alpha"] : $_POST["alpha"]), "alpha", "", true, 3);
 			$save["graph_type_id"] = form_input_validate((isset($item["graph_type_id"]) ? $item["graph_type_id"] : $_POST["graph_type_id"]), "graph_type_id", "", true, 3);
 			$save["cdef_id"] = form_input_validate($_POST["cdef_id"], "cdef_id", "", true, 3);
 			$save["consolidation_function_id"] = form_input_validate((isset($item["consolidation_function_id"]) ? $item["consolidation_function_id"] : $_POST["consolidation_function_id"]), "consolidation_function_id", "", true, 3);
@@ -116,20 +116,20 @@ function form_save() {
 			$save["hard_return"] = form_input_validate(((isset($item["hard_return"]) ? $item["hard_return"] : (isset($_POST["hard_return"]) ? $_POST["hard_return"] : ""))), "hard_return", "", true, 3);
 			$save["gprint_id"] = form_input_validate($_POST["gprint_id"], "gprint_id", "", true, 3);
 			$save["sequence"] = $_POST["sequence"];
-			
+
 			if (!is_error_message()) {
 				$graph_template_item_id = sql_save($save, "graph_templates_item");
-				
+
 				if ($graph_template_item_id) {
 					raise_message(1);
 				}else{
 					raise_message(2);
 				}
 			}
-			
+
 			$_POST["sequence"] = 0;
 		}
-		
+
 		if (is_error_message()) {
 			header("Location: graphs.php?action=item_edit&graph_template_item_id=" . (empty($graph_template_item_id) ? $_POST["graph_template_item_id"] : $graph_template_item_id) . "&id=" . $_POST["local_graph_id"]);
 			exit;
@@ -141,15 +141,20 @@ function form_save() {
 }
 
 /* -----------------------
-    item - Graph Items 
+    item - Graph Items
    ----------------------- */
 
 function item_movedown() {
 	global $graph_item_types;
-	
+
+	/* ================= input validation ================= */
+	input_validate_input_number(get_request_var("id"));
+	input_validate_input_number(get_request_var("local_graph_id"));
+	/* ==================================================== */
+
 	$arr = get_graph_group($_GET["id"]);
 	$next_id = get_graph_parent($_GET["id"], "next");
-	
+
 	if ((!empty($next_id)) && (isset($arr{$_GET["id"]}))) {
 		move_graph_group($_GET["id"], $arr, $next_id, "next");
 	}elseif (ereg("(GPRINT|VRULE|HRULE|COMMENT)", $graph_item_types{db_fetch_cell("select graph_type_id from graph_templates_item where id=" . $_GET["id"])})) {
@@ -159,10 +164,15 @@ function item_movedown() {
 
 function item_moveup() {
 	global $graph_item_types;
-	
+
+	/* ================= input validation ================= */
+	input_validate_input_number(get_request_var("id"));
+	input_validate_input_number(get_request_var("local_graph_id"));
+	/* ==================================================== */
+
 	$arr = get_graph_group($_GET["id"]);
 	$previous_id = get_graph_parent($_GET["id"], "previous");
-	
+
 	if ((!empty($previous_id)) && (isset($arr{$_GET["id"]}))) {
 		move_graph_group($_GET["id"], $arr, $previous_id, "previous");
 	}elseif (ereg("(GPRINT|VRULE|HRULE|COMMENT)", $graph_item_types{db_fetch_cell("select graph_type_id from graph_templates_item where id=" . $_GET["id"])})) {
@@ -171,53 +181,102 @@ function item_moveup() {
 }
 
 function item_remove() {
-	db_execute("delete from graph_templates_item where id=" . $_GET["id"]);	
+	/* ================= input validation ================= */
+	input_validate_input_number(get_request_var("id"));
+	/* ==================================================== */
+
+	db_execute("delete from graph_templates_item where id=" . $_GET["id"]);
 }
 
 function item_edit() {
 	global $colors, $struct_graph_item, $graph_item_types, $consolidation_functions;
-	
-	if (!empty($_GET["id"])) {
-		$template_item = db_fetch_row("select * from graph_templates_item where id=" . $_GET["id"]);
-		$host_id = db_fetch_cell("select host_id from graph_local where id=" . $_GET["local_graph_id"]);
+
+	/* ================= input validation ================= */
+	input_validate_input_number(get_request_var_request("id"));
+	input_validate_input_number(get_request_var_request("host_id"));
+	input_validate_input_number(get_request_var_request("graph_template_id"));
+	input_validate_input_number(get_request_var_request("local_graph_id"));
+	input_validate_input_number(get_request_var_request("host_id"));
+	input_validate_input_number(get_request_var_request("data_template_id"));
+	/* ==================================================== */
+
+	/* remember these search fields in session vars so we don't have to keep passing them around */
+	load_current_session_value("local_graph_id", "sess_local_graph_id", "");
+	load_current_session_value("host_id", "sess_ds_host_id", "-1");
+	load_current_session_value("data_template_id", "sess_data_template_id", "-1");
+
+	$host = db_fetch_row("select hostname from host where id=" . $_REQUEST["host_id"]);
+
+	html_start_box("<strong>Data Sources</strong> [host: " . (empty($host["hostname"]) ? "No Host" : $host["hostname"]) . "]", "100%", $colors["header"], "3", "center", "");
+
+	include("./include/html/inc_graph_items_filter_table.php");
+
+	html_end_box();
+
+	if ($_REQUEST["host_id"] == "-1") {
+		$sql_where = "";
+	}elseif ($_REQUEST["host_id"] == "0") {
+		$sql_where = " data_local.host_id=0 and ";
+	}elseif (!empty($_REQUEST["host_id"])) {
+		$sql_where = " data_local.host_id=" . $_REQUEST["host_id"] . " and ";
 	}
-	
-	$header_label = "[edit graph: " . db_fetch_cell("select title_cache from graph_templates_graph where local_graph_id=" . $_GET["local_graph_id"]) . "]";
-	
-	start_box("<strong>Graph Items</strong> $header_label", "98%", $colors["header"], "3", "center", "");
-	
+
+	if ($_REQUEST["data_template_id"] == "-1") {
+		$sql_where .= "";
+	}elseif ($_REQUEST["data_template_id"] == "0") {
+		$sql_where .= " data_local.data_template_id=0 and ";
+	}elseif (!empty($_REQUEST["data_template_id"])) {
+		$sql_where .= " data_local.data_template_id=" . $_REQUEST["data_template_id"] . " and ";
+	}
+
+	if (!empty($_REQUEST["id"])) {
+		$template_item = db_fetch_row("select * from graph_templates_item where id=" . $_REQUEST["id"]);
+		$host_id = db_fetch_cell("select host_id from graph_local where id=" . $_REQUEST["local_graph_id"]);
+	}
+
+	$header_label = "[edit graph: " . db_fetch_cell("select title_cache from graph_templates_graph where local_graph_id=" . $_REQUEST["local_graph_id"]) . "]";
+
+	html_start_box("<strong>Graph Items</strong> $header_label", "100%", $colors["header"], "3", "center", "");
+
 	/* by default, select the LAST DS chosen to make everyone's lives easier */
-	if (!empty($_GET["local_graph_id"])) {
-		$default = db_fetch_row("select task_item_id from graph_templates_item where local_graph_id=" . $_GET["local_graph_id"] . " order by sequence DESC");
-		
+	if (!empty($_REQUEST["local_graph_id"])) {
+		$default = db_fetch_row("select task_item_id from graph_templates_item where local_graph_id=" . $_REQUEST["local_graph_id"] . " order by sequence DESC");
+
 		if (sizeof($default) > 0) {
 			$struct_graph_item["task_item_id"]["default"] = $default["task_item_id"];
 		}else{
 			$struct_graph_item["task_item_id"]["default"] = 0;
 		}
-		
+
 		/* modifications to the default graph items array */
 		$struct_graph_item["task_item_id"]["sql"] = "select
-			CONCAT_WS('',case when host.description is null then 'No Host' when host.description is not null then host.description end,' - ',data_template_data.name_cache,' (',data_template_rrd.data_source_name,')') as name,
-			data_template_rrd.id 
-			from data_template_data,data_template_rrd,data_local 
-			left join host on data_local.host_id=host.id
-			where data_template_rrd.local_data_id=data_local.id 
-			and data_template_data.local_data_id=data_local.id
-			" . (((!empty($host_id)) || (!empty($_GET["host_id"]))) ? (!empty($host_id) ? " and data_local.host_id=$host_id" : " and data_local.host_id=" . $_GET["host_id"]) : "") . "
-			order by name";
+			CONCAT_WS('',data_template_data.name_cache,' (',data_template_rrd.data_source_name,')') as name,
+			data_template_rrd.id
+			from (data_template_data,data_template_rrd,data_local)
+			left join host on (data_local.host_id=host.id)
+			where data_template_rrd.local_data_id=data_local.id
+			and data_template_data.local_data_id=data_local.id ";
+		/* Make sure we don't limit the list so that the selected DS isn't in the list in edit mode */
+		if (strlen($sql_where) > 0) {
+			$sql_where = substr($sql_where,0,-5);
+			if (!empty($_REQUEST["id"])) {
+				$struct_graph_item["task_item_id"]["sql"] .= " and ((" . $sql_where .  ") or (data_template_rrd.id = " .  $template_item["task_item_id"] . "))";
+			} else {
+				$struct_graph_item["task_item_id"]["sql"] .= " and (" . $sql_where . ")";
+			}
+		}
+		$struct_graph_item["task_item_id"]["sql"] .= " order by name";
 	}
-	
+
 	$form_array = array();
-	
+
 	while (list($field_name, $field_array) = each($struct_graph_item)) {
 		$form_array += array($field_name => $struct_graph_item[$field_name]);
-		
+
 		$form_array[$field_name]["value"] = (isset($template_item) ? $template_item[$field_name] : "");
 		$form_array[$field_name]["form_id"] = (isset($template_item) ? $template_item["id"] : "0");
-		
 	}
-	
+
 	draw_edit_form(
 		array(
 			"config" => array(
@@ -225,16 +284,43 @@ function item_edit() {
 			"fields" => $form_array
 			)
 		);
-	
-	form_hidden_id("local_graph_id",$_GET["local_graph_id"]);
-	form_hidden_id("graph_template_item_id",(isset($template_item) ? $template_item["id"] : "0"));
-	form_hidden_id("local_graph_template_item_id",(isset($template_item) ? $template_item["local_graph_template_item_id"] : "0"));
-	form_hidden_id("graph_template_id",(isset($template_item) ? $template_item["graph_template_id"] : "0"));
-	form_hidden_id("sequence",(isset($template_item) ? $template_item["sequence"] : "0"));
-	form_hidden_id("_graph_type_id",(isset($template_item) ? $template_item["graph_type_id"] : "0"));
-	form_hidden_box("save_component_item","1","");
-	
-	end_box();
-	
-	form_save_button("graphs.php?action=graph_edit&id=" . $_GET["local_graph_id"]);
+
+	form_hidden_box("local_graph_id", $_REQUEST["local_graph_id"], "0");
+	form_hidden_box("graph_template_item_id", (isset($template_item) ? $template_item["id"] : "0"), "");
+	form_hidden_box("local_graph_template_item_id", (isset($template_item) ? $template_item["local_graph_template_item_id"] : "0"), "");
+	form_hidden_box("graph_template_id", (isset($template_item) ? $template_item["graph_template_id"] : "0"), "");
+	form_hidden_box("sequence", (isset($template_item) ? $template_item["sequence"] : "0"), "");
+	form_hidden_box("_graph_type_id", (isset($template_item) ? $template_item["graph_type_id"] : "0"), "");
+	form_hidden_box("save_component_item", "1", "");
+	form_hidden_box("invisible_alpha", $form_array["alpha"]["value"], "FF");
+	form_hidden_box("rrdtool_version", read_config_option("rrdtool_version"), "");
+
+	html_end_box();
+
+	form_save_button("graphs.php?action=graph_edit&id=" . $_REQUEST["local_graph_id"]);
 }
+
+//Now we need some javascript to make it dynamic
+?>
+<script language="JavaScript">
+
+dynamic();
+
+function dynamic() {
+	//alert("RRDTool Version is '" + document.getElementById('rrdtool_version').value + "'");
+	//alert("Color is '" + document.getElementById('color_id').value + "'");
+	document.getElementById('alpha').disabled=true;
+	if ((document.getElementById('rrdtool_version').value != 'rrd-1.0.x') &&
+		(document.getElementById('color_id').value != 0)) {
+		document.getElementById('alpha').disabled=false;
+	}
+}
+
+function changeColorId() {
+	//alert("Selected Color Index is '" + document.getElementById('color_id').selectedIndex + "'");
+	if ((document.getElementById('rrdtool_version').value != 'rrd-1.0.x') &&
+		(document.getElementById('color_id').selectedIndex != 0)) {
+		document.getElementById('alpha').disabled=false;
+	}
+}
+</script>
