@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004 Ian Berry                                            |
+ | Copyright (C) 2004-2007 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -13,23 +13,21 @@
  | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
  | GNU General Public License for more details.                            |
  +-------------------------------------------------------------------------+
- | cacti: a php-based graphing solution                                    |
+ | Cacti: The Complete RRDTool-based Graphing Solution                     |
  +-------------------------------------------------------------------------+
- | Most of this code has been designed, written and is maintained by       |
- | Ian Berry. See about.php for specific developer credit. Any questions   |
- | or comments regarding this code should be directed to:                  |
- | - iberry@raxnet.net                                                     |
+ | This code is designed, written, and maintained by the Cacti Group. See  |
+ | about.php and/or the AUTHORS file for specific developer information.   |
  +-------------------------------------------------------------------------+
- | - raXnet - http://www.raxnet.net/                                       |
+ | http://www.cacti.net/                                                   |
  +-------------------------------------------------------------------------+
 */
 
-include("../include/config.php");
+include("../include/global.php");
 
 /* allow the upgrade script to run for as long as it needs to */
 ini_set("max_execution_time", "0");
 
-$cacti_versions = array("0.8", "0.8.1", "0.8.2", "0.8.2a", "0.8.3", "0.8.3a", "0.8.4", "0.8.5", "0.8.5a", "0.8.6", "0.8.6a", "0.8.6b");
+$cacti_versions = array("0.8", "0.8.1", "0.8.2", "0.8.2a", "0.8.3", "0.8.3a", "0.8.4", "0.8.5", "0.8.5a", "0.8.6", "0.8.6a", "0.8.6b", "0.8.6c", "0.8.6d", "0.8.6e", "0.8.6f", "0.8.6g", "0.8.6h", "0.8.6i", "0.8.6j", "0.8.7", "0.8.7a");
 
 $old_cacti_version = db_fetch_cell("select cacti from version");
 
@@ -39,7 +37,7 @@ $old_version_index = array_search($old_cacti_version, $cacti_versions);
 /* do a version check */
 if ($old_cacti_version == $config["cacti_version"]) {
 	print "	<p style='font-family: Verdana, Arial; font-size: 16px; font-weight: bold; color: red;'>Error</p>
-		upgrades, this installation is already up-to-date. Click <a href='../index.php'>here</a> to use cacti.</p>";
+		This installation is already up-to-date. Click <a href='../index.php'>here</a> to use cacti.</p>";
 	exit;
 }elseif (ereg("^0\.6", $old_cacti_version)) {
 	print "	<p style='font-family: Verdana, Arial; font-size: 16px; font-weight: bold; color: red;'>Error</p>
@@ -51,7 +49,10 @@ if ($old_cacti_version == $config["cacti_version"]) {
 	print "	<p style='font-family: Verdana, Arial; font-size: 16px; font-weight: bold; color: red;'>Error</p>
 		<p style='font-family: Verdana, Arial; font-size: 12px;'>You have created a new database, but have not yet imported
 		the 'cacti.sql' file. At the command line, execute the following to continue:</p>
-		<p><pre>mysql -u $database_username -p $database_default < cacti.sql</pre></p>";
+		<p><pre>mysql -u $database_username -p $database_default < cacti.sql</pre></p>
+		<p>This error may also be generated if the cacti database user does not have correct permissions on the cacti database.
+		Please ensure that the cacti database user has the ability to SELECT, INSERT, DELETE, UPDATE, CREATE, ALTER, DROP, INDEX
+		on the cacti database.</p>";
 	exit;
 }
 
@@ -68,10 +69,15 @@ function db_install_execute($cacti_version, $sql) {
 }
 
 function find_best_path($binary_name) {
-	$search_paths = array("/bin", "/sbin", "/usr/bin", "/usr/sbin", "/usr/local/bin", "/usr/local/sbin");
+	global $config;
+	if ($config["cacti_server_os"] == "win32") {
+		$search_paths = array("c:/usr/bin", "c:/cacti", "c:/rrdtool", "c:/spine", "c:/php", "c:/progra~1/php", "c:/net-snmp/bin", "c:/progra~1/net-snmp/bin", "d:/usr/bin", "d:/net-snmp/bin", "d:/progra~1/net-snmp/bin", "d:/cacti", "d:/rrdtool", "d:/spine", "d:/php", "d:/progra~1/php");
+	}else{
+		$search_paths = array("/bin", "/sbin", "/usr/bin", "/usr/sbin", "/usr/local/bin", "/usr/local/sbin");
+	}
 
 	for ($i=0; $i<count($search_paths); $i++) {
-		if ((file_exists($search_paths[$i] . "/" . $binary_name)) && (is_executable($search_paths[$i] . "/" . $binary_name))) {
+		if ((file_exists($search_paths[$i] . "/" . $binary_name)) && (is_readable($search_paths[$i] . "/" . $binary_name))) {
 			return $search_paths[$i] . "/" . $binary_name;
 		}
 	}
@@ -88,15 +94,23 @@ $input["path_rrdtool"] = $settings["path"]["path_rrdtool"];
 if ($config["cacti_server_os"] == "unix") {
 	$which_rrdtool = find_best_path("rrdtool");
 
-	if (!empty($which_rrdtool)) {
-		$input["path_rrdtool"]["default"] = $which_rrdtool;
-	}elseif (read_config_option("path_rrdtool") != "<DEFAULT>") {
+	if (config_value_exists("path_rrdtool")) {
 		$input["path_rrdtool"]["default"] = read_config_option("path_rrdtool");
+	}else if (!empty($which_rrdtool)) {
+		$input["path_rrdtool"]["default"] = $which_rrdtool;
 	}else{
 		$input["path_rrdtool"]["default"] = "/usr/local/bin/rrdtool";
 	}
 }elseif ($config["cacti_server_os"] == "win32") {
-	$input["path_rrdtool"]["default"] = "c:/rrdtool/rrdtool.exe";
+	$which_php = find_best_path("rrdtool.exe");
+
+	if (config_value_exists("path_rrdtool")) {
+		$input["path_rrdtool"]["default"] = read_config_option("path_rrdtool");
+	}else if (!empty($which_rrdtool)) {
+		$input["path_rrdtool"]["default"] = $which_rrdtool;
+	}else{
+		$input["path_rrdtool"]["default"] = "c:/rrdtool/rrdtool.exe";
+	}
 }
 
 /* PHP Binary Path */
@@ -105,51 +119,133 @@ $input["path_php_binary"] = $settings["path"]["path_php_binary"];
 if ($config["cacti_server_os"] == "unix") {
 	$which_php = find_best_path("php");
 
-	if (!empty($which_php)) {
-		$input["path_php_binary"]["default"] = $which_php;
-	}elseif (read_config_option("path_php_binary") != "<DEFAULT>") {
+	if (config_value_exists("path_php_binary")) {
 		$input["path_php_binary"]["default"] = read_config_option("path_php_binary");
+	}else if (!empty($which_php)) {
+		$input["path_php_binary"]["default"] = $which_php;
 	}else{
 		$input["path_php_binary"]["default"] = "/usr/bin/php";
 	}
 }elseif ($config["cacti_server_os"] == "win32") {
-	$input["path_php_binary"]["default"] = "c:/php/php.exe";
+	$which_php = find_best_path("php.exe");
+
+	if (config_value_exists("path_php_binary")) {
+		$input["path_php_binary"]["default"] = read_config_option("path_php_binary");
+	}else if (!empty($which_php)) {
+		$input["path_php_binary"]["default"] = $which_php;
+	}else{
+		$input["path_php_binary"]["default"] = "c:/php/php.exe";
+	}
 }
 
 /* snmpwalk Binary Path */
-if ($config["cacti_server_os"] == "unix") {
-	$input["path_snmpwalk"] = $settings["path"]["path_snmpwalk"];
+$input["path_snmpwalk"] = $settings["path"]["path_snmpwalk"];
 
+if ($config["cacti_server_os"] == "unix") {
 	$which_snmpwalk = find_best_path("snmpwalk");
 
-	if (!empty($which_snmpwalk)) {
-		$input["path_snmpwalk"]["default"] = $which_snmpwalk;
-	}elseif (read_config_option("path_snmpwalk") != "<DEFAULT>") {
+	if (config_value_exists("path_snmpwalk")) {
 		$input["path_snmpwalk"]["default"] = read_config_option("path_snmpwalk");
+	}else if (!empty($which_snmpwalk)) {
+		$input["path_snmpwalk"]["default"] = $which_snmpwalk;
 	}else{
 		$input["path_snmpwalk"]["default"] = "/usr/local/bin/snmpwalk";
 	}
+}elseif ($config["cacti_server_os"] == "win32") {
+	$which_snmpwalk = find_best_path("snmpwalk.exe");
+
+	if (config_value_exists("path_snmpwalk")) {
+		$input["path_snmpwalk"]["default"] = read_config_option("path_snmpwalk");
+	}else if (!empty($which_snmpwalk)) {
+		$input["path_snmpwalk"]["default"] = $which_snmpwalk;
+	}else{
+		$input["path_snmpwalk"]["default"] = "c:/net-snmp/bin/snmpwalk.exe";
+	}
 }
 
 /* snmpget Binary Path */
-if ($config["cacti_server_os"] == "unix") {
-	$input["path_snmpget"] = $settings["path"]["path_snmpget"];
+$input["path_snmpget"] = $settings["path"]["path_snmpget"];
 
+if ($config["cacti_server_os"] == "unix") {
 	$which_snmpget = find_best_path("snmpget");
 
-	if (!empty($which_snmpget)) {
-		$input["path_snmpget"]["default"] = $which_snmpget;
-	}elseif (read_config_option("path_snmpget") != "<DEFAULT>") {
+	if (config_value_exists("path_snmpget")) {
 		$input["path_snmpget"]["default"] = read_config_option("path_snmpget");
+	}else if (!empty($which_snmpget)) {
+		$input["path_snmpget"]["default"] = $which_snmpget;
 	}else{
 		$input["path_snmpget"]["default"] = "/usr/local/bin/snmpget";
 	}
+}elseif ($config["cacti_server_os"] == "win32") {
+	$which_snmpget = find_best_path("snmpget.exe");
+
+	if (config_value_exists("path_snmpget")) {
+		$input["path_snmpget"]["default"] = read_config_option("path_snmpget");
+	}else if (!empty($which_snmpget)) {
+		$input["path_snmpget"]["default"] = $which_snmpget;
+	}else{
+		$input["path_snmpget"]["default"] = "c:/net-snmp/bin/snmpget.exe";
+	}
+}
+
+/* snmpbulkwalk Binary Path */
+$input["path_snmpbulkwalk"] = $settings["path"]["path_snmpbulkwalk"];
+
+if ($config["cacti_server_os"] == "unix") {
+	$which_snmpbulkwalk = find_best_path("snmpbulkwalk");
+
+	if (config_value_exists("path_snmpbulkwalk")) {
+		$input["path_snmpbulkwalk"]["default"] = read_config_option("path_snmpbulkwalk");
+	}else if (!empty($which_snmpbulkwalk)) {
+		$input["path_snmpbulkwalk"]["default"] = $which_snmpbulkwalk;
+	}else{
+		$input["path_snmpbulkwalk"]["default"] = "/usr/local/bin/snmpbulkwalk";
+	}
+}elseif ($config["cacti_server_os"] == "win32") {
+	$which_snmpbulkwalk = find_best_path("snmpbulkwalk.exe");
+
+	if (config_value_exists("path_snmpbulkwalk")) {
+		$input["path_snmpbulkwalk"]["default"] = read_config_option("path_snmpbulkwalk");
+	}else if (!empty($which_snmpbulkwalk)) {
+		$input["path_snmpbulkwalk"]["default"] = $which_snmpbulkwalk;
+	}else{
+		$input["path_snmpbulkwalk"]["default"] = "c:/net-snmp/bin/snmpbulkwalk.exe";
+	}
+}
+
+/* snmpgetnext Binary Path */
+$input["path_snmpgetnext"] = $settings["path"]["path_snmpgetnext"];
+
+if ($config["cacti_server_os"] == "unix") {
+	$which_snmpgetnext = find_best_path("snmpgetnext");
+
+	if (config_value_exists("path_snmpgetnext")) {
+		$input["path_snmpgetnext"]["default"] = read_config_option("path_snmpgetnext");
+	}else if (!empty($which_snmpgetnext)) {
+		$input["path_snmpgetnext"]["default"] = $which_snmpgetnext;
+	}else{
+		$input["path_snmpgetnext"]["default"] = "/usr/local/bin/snmpgetnext";
+	}
+}elseif ($config["cacti_server_os"] == "win32") {
+	$which_snmpgetnext = find_best_path("snmpgetnext.exe");
+
+	if (config_value_exists("path_snmpgetnext")) {
+		$input["path_snmpgetnext"]["default"] = read_config_option("path_snmpgetnext");
+	}else if (!empty($which_snmpgetnext)) {
+		$input["path_snmpgetnext"]["default"] = $which_snmpgetnext;
+	}else{
+		$input["path_snmpgetnext"]["default"] = "c:/net-snmp/bin/snmpgetnext.exe";
+	}
 }
 
 /* log file path */
 $input["path_cactilog"] = $settings["path"]["path_cactilog"];
 $input["path_cactilog"]["description"] = "The path to your Cacti log file.";
-$input["path_cactilog"]["default"] = $config["base_path"] . "/log/cacti.log";
+if (config_value_exists("path_cactilog")) {
+	$input["path_cactilog"]["default"] = read_config_option("path_cactilog");
+} else {
+	$input["path_cactilog"]["default"] = $config["base_path"] . "/log/cacti.log";
+}
 
 /* SNMP Version */
 if ($config["cacti_server_os"] == "unix") {
@@ -157,6 +253,23 @@ if ($config["cacti_server_os"] == "unix") {
 	$input["snmp_version"]["default"] = "net-snmp";
 }
 
+/* RRDTool Version */
+if ((file_exists($input["path_rrdtool"]["default"])) && (($config["cacti_server_os"] == "win32") || (is_executable($input["path_rrdtool"]["default"]))) ) {
+	$input["rrdtool_version"] = $settings["general"]["rrdtool_version"];
+
+	$out_array = array();
+
+	exec($input["path_rrdtool"]["default"], $out_array);
+
+	if (sizeof($out_array) > 0) {
+		if (ereg("^RRDtool 1\.2", $out_array[0])) {
+			$input["rrdtool_version"]["default"] = "rrd-1.2.x";
+		}else if (ereg("^RRDtool 1\.0\.", $out_array[0])) {
+			$input["rrdtool_version"]["default"] = "rrd-1.0.x";
+		}
+	}
+}
+
 /* default value for this variable */
 if (!isset($_REQUEST["install_type"])) {
 	$_REQUEST["install_type"] = 0;
@@ -199,7 +312,7 @@ if ($_REQUEST["step"] == "4") {
 	/* get all items on the form and write values for them  */
 	while (list($name, $array) = each($input)) {
 		if (isset($_POST[$name])) {
-			db_execute("update settings set value='" . $_POST[$name] . "' where name='$name'");
+			db_execute("replace into settings (name,value) values ('$name','" . $_POST[$name] . "')");
 		}
 	}
 
@@ -209,7 +322,11 @@ if ($_REQUEST["step"] == "4") {
 	kill_session_var("sess_host_cache_array");
 
 	/* just in case we have hard drive graphs to deal with */
-	run_data_query(db_fetch_cell("select id from host where hostname='127.0.0.1'"), 6);
+	$host_id = db_fetch_cell("select id from host where hostname='127.0.0.1'");
+
+	if (!empty($host_id)) {
+		run_data_query($host_id, 6);
+	}
 
 	/* it's always a good idea to re-populate the poller cache to make sure everything is refreshed and
 	up-to-date */
@@ -251,24 +368,33 @@ if ($_REQUEST["step"] == "4") {
 		}elseif ($cacti_versions[$i] == "0.8.5") {
 			include ("0_8_4_to_0_8_5.php");
 			upgrade_to_0_8_5();
-		}elseif ($cacti_versions[$i] == "0.8.5a") {
-			/* no database upgrades for 0.8.5 -> 0.8.5a */
 		}elseif ($cacti_versions[$i] == "0.8.6") {
 			include ("0_8_5a_to_0_8_6.php");
 			upgrade_to_0_8_6();
 		}elseif ($cacti_versions[$i] == "0.8.6a") {
 			include ("0_8_6_to_0_8_6a.php");
 			upgrade_to_0_8_6a();
-
-			if ($old_cacti_version == "0.8.6") {
-				/* no database upgrades for 0.8.6 -> 0.8.6a */
-				$_REQUEST["step"] = "3";
-			}
-		}elseif ($cacti_versions[$i] == "0.8.6b") {
-			if ($old_cacti_version == "0.8.6a") {
-				/* no database upgrades for 0.8.6a -> 0.8.6b */
-				$_REQUEST["step"] = "3";
-			}
+		}elseif ($cacti_versions[$i] == "0.8.6d") {
+			include ("0_8_6c_to_0_8_6d.php");
+			upgrade_to_0_8_6d();
+		}elseif ($cacti_versions[$i] == "0.8.6e") {
+			include ("0_8_6d_to_0_8_6e.php");
+			upgrade_to_0_8_6e();
+		}elseif ($cacti_versions[$i] == "0.8.6g") {
+			include ("0_8_6f_to_0_8_6g.php");
+			upgrade_to_0_8_6g();
+		}elseif ($cacti_versions[$i] == "0.8.6h") {
+			include ("0_8_6g_to_0_8_6h.php");
+			upgrade_to_0_8_6h();
+		}elseif ($cacti_versions[$i] == "0.8.6i") {
+			include ("0_8_6h_to_0_8_6i.php");
+			upgrade_to_0_8_6i();
+		}elseif ($cacti_versions[$i] == "0.8.7") {
+			include ("0_8_6j_to_0_8_7.php");
+			upgrade_to_0_8_7();
+		}elseif ($cacti_versions[$i] == "0.8.7a") {
+			include ("0_8_7_to_0_8_7a.php");
+			upgrade_to_0_8_7a();
 		}
 	}
 }
@@ -376,7 +502,8 @@ if ($_REQUEST["step"] == "4") {
 								the results ('FOUND' or 'NOT FOUND') so they can be displayed on the form */
 								$form_check_string = "";
 
-								if ($array["method"] == "textbox") {
+								if (($array["method"] == "textbox") ||
+									($array["method"] == "filepath")) {
 									if (@file_exists($current_value)) {
 										$form_check_string = "<font color='#008000'>[FOUND]</font> ";
 									}else{
@@ -399,6 +526,9 @@ if ($_REQUEST["step"] == "4") {
 								case 'textbox':
 									form_text_box($name, $current_value, "", "", "40", "text");
 									break;
+								case 'filepath':
+									form_filepath_box($name, $current_value, "", "", "40", "text");
+									break;
 								case 'drop_array':
 									form_dropdown($name, $array["array"], "", "", $current_value, "", "");
 									break;
