@@ -1,26 +1,24 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004 Ian Berry                                            |
+ | Copyright (C) 2004-2007 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
  | as published by the Free Software Foundation; either version 2          |
  | of the License, or (at your option) any later version.                  |
  |                                                                         |
- | This program is distributed in the hope that it will be useful,
+ | This program is distributed in the hope that it will be useful,         |
  | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
  | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
  | GNU General Public License for more details.                            |
  +-------------------------------------------------------------------------+
- | cacti: a php-based graphing solution                                    |
+ | Cacti: The Complete RRDTool-based Graphing Solution                     |
  +-------------------------------------------------------------------------+
- | Most of this code has been designed, written and is maintained by       |
- | Ian Berry. See about.php for specific developer credit. Any questions   |
- | or comments regarding this code should be directed to:                  |
- | - iberry@raxnet.net                                                     |
+ | This code is designed, written, and maintained by the Cacti Group. See  |
+ | about.php and/or the AUTHORS file for specific developer information.   |
  +-------------------------------------------------------------------------+
- | - raXnet - http://www.raxnet.net/                                       |
+ | http://www.cacti.net/                                                   |
  +-------------------------------------------------------------------------+
 */
 
@@ -28,9 +26,10 @@ function run_data_query($host_id, $snmp_query_id) {
 	global $config;
 
 	include_once($config["library_path"] . "/poller.php");
+	include_once($config["library_path"] . "/utility.php");
 
 	debug_log_insert("data_query", "Running data query [$snmp_query_id].");
-	$type_id = db_fetch_cell("select data_input.type_id from snmp_query,data_input where snmp_query.data_input_id=data_input.id and snmp_query.id=$snmp_query_id");
+	$type_id = db_fetch_cell("select data_input.type_id from (snmp_query,data_input) where snmp_query.data_input_id=data_input.id and snmp_query.id=$snmp_query_id");
 
 	if ($type_id == DATA_INPUT_TYPE_SNMP_QUERY) {
 		debug_log_insert("data_query", "Found type = '3' [snmp query].");
@@ -54,6 +53,9 @@ function run_data_query($host_id, $snmp_query_id) {
 	/* update the the "local" data query cache */
 	update_data_query_cache($host_id, $snmp_query_id);
 
+	/* update the poller cache */
+	update_poller_cache_from_query($host_id, $snmp_query_id);
+
 	return (isset($result) ? $result : true);
 }
 
@@ -79,7 +81,8 @@ function get_data_query_array($snmp_query_id) {
 function query_script_host($host_id, $snmp_query_id) {
 	$script_queries = get_data_query_array($snmp_query_id);
 
-	if ($script_queries == false) {
+	/* invalid xml check */
+	if ((!is_array($script_queries)) || (sizeof($script_queries) == 0)) {
 		debug_log_insert("data_query", "Error parsing XML file into an array.");
 		return false;
 	}
@@ -130,11 +133,30 @@ function query_snmp_host($host_id, $snmp_query_id) {
 
 	include_once($config["library_path"] . "/snmp.php");
 
-	$host = db_fetch_row("select hostname,snmp_community,snmp_version,snmp_username,snmp_password,snmp_port,snmp_timeout from host where id=$host_id");
+	$host = db_fetch_row("SELECT
+		hostname,
+		snmp_community,
+		snmp_version,
+		snmp_username,
+		snmp_password,
+		snmp_auth_protocol,
+		snmp_priv_passphrase,
+		snmp_priv_protocol,
+		snmp_context,
+		snmp_port,
+		snmp_timeout
+		FROM host
+		WHERE id='$host_id'");
 
 	$snmp_queries = get_data_query_array($snmp_query_id);
 
-	if ((empty($host["hostname"])) || ($snmp_queries == false)) {
+	if ($host["hostname"] == "") {
+		debug_log_insert("data_query", "Invalid host_id: $host_id");
+		return false;
+	}
+
+	/* invalid xml check */
+	if ((!is_array($snmp_queries)) || (sizeof($snmp_queries) == 0)) {
 		debug_log_insert("data_query", "Error parsing XML file into an array.");
 		return false;
 	}
@@ -142,7 +164,10 @@ function query_snmp_host($host_id, $snmp_query_id) {
 	debug_log_insert("data_query", "XML file parsed ok.");
 
 	/* fetch specified index at specified OID */
-	$snmp_index = cacti_snmp_walk($host["hostname"], $host["snmp_community"], $snmp_queries["oid_index"], $host["snmp_version"], $host["snmp_username"], $host["snmp_password"], $host["snmp_port"], $host["snmp_timeout"], SNMP_WEBUI);
+	$snmp_index = cacti_snmp_walk($host["hostname"], $host["snmp_community"], $snmp_queries["oid_index"],
+		$host["snmp_version"], $host["snmp_username"], $host["snmp_password"],
+		$host["snmp_auth_protocol"], $host["snmp_priv_passphrase"], $host["snmp_priv_protocol"],
+		$host["snmp_context"], $host["snmp_port"], $host["snmp_timeout"], read_config_option("snmp_retries"), SNMP_WEBUI);
 
 	debug_log_insert("data_query", "Executing SNMP walk for list of indexes @ '" . $snmp_queries["oid_index"] . "'");
 
@@ -152,56 +177,90 @@ function query_snmp_host($host_id, $snmp_query_id) {
 		return false;
 	}
 
-	db_execute("delete from host_snmp_cache where host_id=$host_id and snmp_query_id=$snmp_query_id");
+	/* the last octet of the oid is the index by default */
+	$index_parse_regexp = '.*\.([0-9]+)$';
+
+	/* parse the index if required */
+	if (isset($snmp_queries["oid_index_parse"])) {
+		$index_parse_regexp = str_replace("OID/REGEXP:", "", $snmp_queries["oid_index_parse"]);
+
+		for ($i=0; $i<sizeof($snmp_index); $i++) {
+			$snmp_index[$i]["value"] = ereg_replace($index_parse_regexp, "\\1", $snmp_index[$i]["oid"]);
+		}
+	}
+
+	db_execute("delete from host_snmp_cache where host_id='$host_id' and snmp_query_id='$snmp_query_id'");
 
 	while (list($field_name, $field_array) = each($snmp_queries["fields"])) {
-		if (($field_array["method"] == "get") && ($field_array["direction"] == "input")) {
+		if ((!isset($field_array["oid"])) && ($field_array["source"] == "index")) {
+			for ($i=0; $i<sizeof($snmp_index); $i++) {
+				debug_log_insert("data_query", "Inserting index data [value='" . $snmp_index[$i]["value"] . "']");
+
+				db_execute("replace into host_snmp_cache
+					(host_id, snmp_query_id, field_name, field_value, snmp_index, oid)
+					values ('$host_id','$snmp_query_id','$field_name','" . $snmp_index[$i]["value"] . "','" . $snmp_index[$i]["value"] . "','')");
+			}
+		}else if (($field_array["method"] == "get") && ($field_array["direction"] == "input")) {
 			debug_log_insert("data_query", "Located input field '$field_name' [get]");
 
 			if ($field_array["source"] == "value") {
-				for ($i=0;($i<sizeof($snmp_index));$i++) {
+				for ($i=0; $i<sizeof($snmp_index); $i++) {
 					$oid = $field_array["oid"] .  "." . $snmp_index[$i]["value"];
 
-					$value = cacti_snmp_get($host["hostname"], $host["snmp_community"], $oid, $host["snmp_version"], $host["snmp_username"], $host["snmp_password"], $host["snmp_port"], $host["snmp_timeout"], SNMP_WEBUI);
+					$value = cacti_snmp_get($host["hostname"], $host["snmp_community"], $oid,
+						$host["snmp_version"], $host["snmp_username"], $host["snmp_password"],
+						$host["snmp_auth_protocol"], $host["snmp_priv_passphrase"], $host["snmp_priv_protocol"],
+						$host["snmp_context"], $host["snmp_port"], $host["snmp_timeout"], SNMP_WEBUI);
 
 					debug_log_insert("data_query", "Executing SNMP get for data @ '$oid' [value='$value']");
 
 					db_execute("replace into host_snmp_cache
 						(host_id,snmp_query_id,field_name,field_value,snmp_index,oid)
-						values ($host_id,$snmp_query_id,'$field_name','$value'," . $snmp_index[$i]["value"] . ",'$oid')");
+						values ('$host_id','$snmp_query_id','$field_name','$value','" . $snmp_index[$i]["value"] . "','$oid')");
 				}
 			}
-		}elseif (($field_array["method"] == "walk") && ($field_array["direction"] == "input")) {
+		}else if (($field_array["method"] == "walk") && ($field_array["direction"] == "input")) {
 			debug_log_insert("data_query", "Located input field '$field_name' [walk]");
 
-			$snmp_data = cacti_snmp_walk($host["hostname"], $host["snmp_community"], $field_array["oid"], $host["snmp_version"], $host["snmp_username"], $host["snmp_password"], $host["snmp_port"], $host["snmp_timeout"], SNMP_WEBUI);
+			$snmp_data = array();
+			$snmp_data = cacti_snmp_walk($host["hostname"], $host["snmp_community"], $field_array["oid"],
+				$host["snmp_version"], $host["snmp_username"], $host["snmp_password"],
+				$host["snmp_auth_protocol"], $host["snmp_priv_passphrase"], $host["snmp_priv_protocol"],
+				$host["snmp_context"], $host["snmp_port"], $host["snmp_timeout"], read_config_option("snmp_retries"), SNMP_WEBUI);
 
 			debug_log_insert("data_query", "Executing SNMP walk for data @ '" . $field_array["oid"] . "'");
 
 			if ($field_array["source"] == "value") {
-				for ($i=0;($i<sizeof($snmp_data));$i++) {
-					$snmp_index = ereg_replace('.*\.([0-9]+)$', "\\1", $snmp_data[$i]["oid"]);
+				for ($i=0; $i<sizeof($snmp_data); $i++) {
+					$snmp_index = ereg_replace($index_parse_regexp, "\\1", $snmp_data[$i]["oid"]);
 
 					$oid = $field_array["oid"] . ".$snmp_index";
 
 					if ($field_name == "ifOperStatus") {
-						if ($snmp_data[$i]["value"] == "down(2)") $snmp_data[$i]["value"] = "Down";
-						if ($snmp_data[$i]["value"] == "up(1)") $snmp_data[$i]["value"] = "Up";
+						if ((substr_count(strtolower($snmp_data[$i]["value"]), "down")) ||
+							($snmp_data[$i]["value"] == "2")) {
+							$snmp_data[$i]["value"] = "Down";
+						}else if ((substr_count(strtolower($snmp_data[$i]["value"]), "up")) ||
+							($snmp_data[$i]["value"] == "1")) {
+							$snmp_data[$i]["value"] = "Up";
+						}else{
+							$snmp_data[$i]["value"] = "Testing";
+						}
 					}
 
 					debug_log_insert("data_query", "Found item [$field_name='" . $snmp_data[$i]["value"] . "'] index: $snmp_index [from value]");
 
 					db_execute("replace into host_snmp_cache
 						(host_id,snmp_query_id,field_name,field_value,snmp_index,oid)
-						values ($host_id,$snmp_query_id,'$field_name','" . $snmp_data[$i]["value"] . "',$snmp_index,'$oid')");
+						values ('$host_id','$snmp_query_id','$field_name','" . $snmp_data[$i]["value"] . "','$snmp_index','$oid')");
 				}
-			}elseif (ereg("^OID/REGEXP:", $field_array["source"])) {
-				for ($i=0;($i<sizeof($snmp_data));$i++) {
-					$value = ereg_replace(ereg_replace("^OID/REGEXP:", "", $field_array["source"]), "\\1", $snmp_data[$i]["oid"]);
+			}elseif (substr($field_array["source"], 0, 11) == "OID/REGEXP:") {
+				for ($i=0; $i<sizeof($snmp_data); $i++) {
+					$value = ereg_replace(str_replace("OID/REGEXP:", "", $field_array["source"]), "\\1", $snmp_data[$i]["oid"]);
 
-					if ((!isset($snmp_data[$i]["value"])) || ($snmp_data[$i]["value"] == "")) {
-						/* do nothing */
-					} else {
+					if (isset($snmp_queries["oid_index_parse"])) {
+						$snmp_index = ereg_replace($index_parse_regexp, "\\1", $snmp_data[$i]["oid"]);
+					}else if ((isset($snmp_data[$i]["value"])) && ($snmp_data[$i]["value"] != "")) {
 						$snmp_index = $snmp_data[$i]["value"];
 					}
 
@@ -217,19 +276,19 @@ function query_snmp_host($host_id, $snmp_query_id) {
 
 					db_execute("replace into host_snmp_cache
 						(host_id,snmp_query_id,field_name,field_value,snmp_index,oid)
-						values ($host_id,$snmp_query_id,'$field_name','$value',$snmp_index,'$oid')");
+						values ('$host_id','$snmp_query_id','$field_name','$value','$snmp_index','$oid')");
 				}
-			}elseif (ereg("^VALUE/REGEXP:", $field_array["source"])) {
-				for ($i=0;($i<sizeof($snmp_data));$i++) {
-					$value = ereg_replace(ereg_replace("^VALUE/REGEXP:", "", $field_array["source"]), "\\1", $snmp_data[$i]["value"]);
-					$snmp_index = ereg_replace('.*\.([0-9]+)$', "\\1", $snmp_data[$i]["oid"]);
+			}elseif (substr($field_array["source"], 0, 13) == "VALUE/REGEXP:") {
+				for ($i=0; $i<sizeof($snmp_data); $i++) {
+					$value = ereg_replace(str_replace("VALUE/REGEXP:", "", $field_array["source"]), "\\1", $snmp_data[$i]["value"]);
+					$snmp_index = ereg_replace($index_parse_regexp, "\\1", $snmp_data[$i]["oid"]);
 					$oid = $field_array["oid"] .  "." . $value;
 
 					debug_log_insert("data_query", "Found item [$field_name='$value'] index: $snmp_index [from regexp value parse]");
 
 					db_execute("replace into host_snmp_cache
 						(host_id,snmp_query_id,field_name,field_value,snmp_index,oid)
-						values ($host_id,$snmp_query_id,'$field_name','$value',$snmp_index,'$oid')");
+						values ('$host_id','$snmp_query_id','$field_name','$value','$snmp_index','$oid')");
 				}
 			}
 		}
@@ -268,7 +327,7 @@ function data_query_field_list($data_template_data_id) {
 	$field = db_fetch_assoc("select
 		data_input_fields.type_code,
 		data_input_data.value
-		from data_input_fields,data_input_data
+		from (data_input_fields,data_input_data)
 		where data_input_fields.id=data_input_data.data_input_field_id
 		and data_input_data.data_template_data_id=$data_template_data_id
 		and (data_input_fields.type_code='index_type' or data_input_fields.type_code='index_value' or data_input_fields.type_code='output_type')");
@@ -310,7 +369,7 @@ function decode_data_query_index($encoded_index, $data_query_id, $host_id) {
 }
 
 /* update_data_query_cache - updates the local data query cache for each graph and data
-     source tied to this host/data query
+	source tied to this host/data query
    @arg $host_id - the id of the host to refresh
    @arg $data_query_id - the id of the data query to refresh */
 function update_data_query_cache($host_id, $data_query_id) {
@@ -339,7 +398,7 @@ function update_graph_data_query_cache($local_graph_id) {
 
 	$field = data_query_field_list(db_fetch_cell("select
 		data_template_data.id
-		from graph_templates_item,data_template_rrd,data_template_data
+		from (graph_templates_item,data_template_rrd,data_template_data)
 		where graph_templates_item.task_item_id=data_template_rrd.id
 		and data_template_rrd.local_data_id=data_template_data.local_data_id
 		and graph_templates_item.local_graph_id=$local_graph_id
@@ -408,7 +467,7 @@ function get_formatted_data_query_indexes($host_id, $data_query_id) {
 	$sort_field_data = array_rekey(db_fetch_assoc("select
 		graph_local.snmp_index,
 		host_snmp_cache.field_value
-		from graph_local,host_snmp_cache
+		from (graph_local,host_snmp_cache)
 		where graph_local.host_id=host_snmp_cache.host_id
 		and graph_local.snmp_query_id=host_snmp_cache.snmp_query_id
 		and graph_local.snmp_index=host_snmp_cache.snmp_index
@@ -455,11 +514,27 @@ function get_formatted_data_query_index($host_id, $data_query_id, $data_query_in
 function get_ordered_index_type_list($host_id, $data_query_id, $data_query_index_array = array()) {
 	$raw_xml = get_data_query_array($data_query_id);
 
+	/* invalid xml check */
+	if ((!is_array($raw_xml)) || (sizeof($raw_xml) == 0)) {
+		return array();
+	}
+
 	$xml_outputs = array();
 
 	/* create an SQL string that contains each index in this snmp_index_id */
 	$sql_or = array_to_sql_or($data_query_index_array, "snmp_index");
 
+	/* check for nonunique query parameter, set value */
+	if (isset($raw_xml["index_type"])) {
+		if ($raw_xml["index_type"] == "nonunique") {
+			$nonunique = 1;
+		}else{
+			$nonunique = 0;
+		}
+	} else {
+		$nonunique = 0;
+	}
+
 	/* list each of the input fields for this snmp query */
 	while (list($field_name, $field_array) = each($raw_xml["fields"])) {
 		if ($field_array["direction"] == "input") {
@@ -474,7 +549,7 @@ function get_ordered_index_type_list($host_id, $data_query_id, $data_query_index
 			$aggregate_field_values = array_rekey($field_values, "field_value", "field_value");
 
 			/* fields that contain duplicate or empty values are not suitable to index off of */
-			if (!((sizeof($aggregate_field_values) < sizeof($field_values)) || (in_array("", $aggregate_field_values) == true) || (sizeof($aggregate_field_values) == 0))) {
+			if (!((sizeof($aggregate_field_values) < sizeof($field_values)) || (in_array("", $aggregate_field_values) == true) || (sizeof($aggregate_field_values) == 0)) || ($nonunique)) {
 				array_push($xml_outputs, $field_name);
 			}
 		}
@@ -488,13 +563,13 @@ function get_ordered_index_type_list($host_id, $data_query_id, $data_query_index
 
 		for ($i=0; $i<count($index_order_array); $i++) {
 			if (in_array($index_order_array[$i], $xml_outputs)) {
-				$return_array{$index_order_array[$i]} = $index_order_array[$i] . " (" . $raw_xml["fields"]{$index_order_array[$i]}["name"] . ")";
+				$return_array[] = $index_order_array[$i];
 			}
 		}
 	/* the xml file does not contain a field list, ignore the order */
 	}else{
 		for ($i=0; $i<count($xml_outputs); $i++) {
-			$return_array{$xml_outputs[$i]} = $xml_outputs[$i] . " (" . $raw_xml["fields"]{$xml_outputs[$i]}["name"] . ")";
+			$return_array[] = $xml_outputs[$i];
 		}
 	}
 
@@ -518,7 +593,7 @@ function update_data_query_sort_cache($host_id, $data_query_id) {
 		$sort_field = "";
 	}else{
 		/* grab the first field off the list */
-		list($sort_field, $sort_field_formatted) = each($valid_index_types);
+		$sort_field = $valid_index_types[0];
 	}
 
 	/* substitute variables */
@@ -577,4 +652,4 @@ function get_script_query_path($args, $script_path, $host_id) {
 	return substitute_script_query_path($script_path) . " $extra_arguments";
 }
 
-?>
\ No newline at end of file
+?>
