@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004 Ian Berry                                            |
+ | Copyright (C) 2004-2007 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -13,14 +13,12 @@
  | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
  | GNU General Public License for more details.                            |
  +-------------------------------------------------------------------------+
- | cacti: a php-based graphing solution                                    |
+ | Cacti: The Complete RRDTool-based Graphing Solution                     |
  +-------------------------------------------------------------------------+
- | Most of this code has been designed, written and is maintained by       |
- | Ian Berry. See about.php for specific developer credit. Any questions   |
- | or comments regarding this code should be directed to:                  |
- | - iberry@raxnet.net                                                     |
+ | This code is designed, written, and maintained by the Cacti Group. See  |
+ | about.php and/or the AUTHORS file for specific developer information.   |
  +-------------------------------------------------------------------------+
- | - raXnet - http://www.raxnet.net/                                       |
+ | http://www.cacti.net/                                                   |
  +-------------------------------------------------------------------------+
 */
 
@@ -31,47 +29,90 @@
    @arg $pass - the password to connect to the database server with
    @arg $db_name - the name of the database to connect to
    @arg $db_type - the type of database server to connect to, only 'mysql' is currently supported
+   @arg $retries - the number a time the server should attempt to connect before failing
    @returns - (bool) '1' for success, '0' for error */
-function db_connect_real($host,$user,$pass,$db_name,$db_type) {
+function db_connect_real($host,$user,$pass,$db_name,$db_type, $port = "3306", $retries = 20) {
 	global $cnn_id;
 
+	$i = 0;
 	$cnn_id = NewADOConnection($db_type);
-	if ($cnn_id->Connect($host,$user,$pass,$db_name)) {
-		return(1);
-	}else{
-		die("<br>Cannot connect to MySQL server on '$host'. Please make sure you have specified a valid MySQL
-		database name in 'include/config.php'.");
 
-		return(0);
+	$hostport = $host . ":" . $port;
+
+	while ($i <= $retries) {
+		if ($cnn_id->PConnect($hostport,$user,$pass,$db_name)) {
+			return(1);
+		}
+
+		$i++;
+
+		usleep(40000);
 	}
+
+	die("FATAL: Cannot connect to MySQL server on '$host'. Please make sure you have specified a valid MySQL database name in 'include/config.php'\n");
+
+	return(0);
+}
+
+/* db_close - closes the open connection
+   @returns - the result of the close command */
+function db_close() {
+	global $cnn_id;
+
+	return $cnn_id->Close();
 }
 
 /* db_execute - run an sql query and do not return any output
    @arg $sql - the sql query to execute
+   @arg $log - whether to log error messages, defaults to true
    @returns - '1' for success, '0' for error */
-function db_execute($sql) {
+function db_execute($sql, $log = TRUE) {
 	global $cnn_id;
 
-	if (!$cnn_id) { db_connect(); }
+	$sql = str_replace("  ", " ", str_replace("\n", "", str_replace("\r", "", str_replace("\t", " ", $sql))));
 
-	$query = $cnn_id->Execute($sql);
+	if (read_config_option("log_verbosity") == POLLER_VERBOSITY_DEBUG) {
+		cacti_log("DEBUG: SQL Exec: \"" . $sql . "\"", FALSE);
+	}
 
-	if ($query) {
-		return(1);
-	}else{
-		return(0);
+	$errors = 0;
+	while (1) {
+		$query = $cnn_id->Execute($sql);
+
+		if (($query) || ($cnn_id->ErrorNo() == 1032)) {
+			return(1);
+		}else if (($log) || (read_config_option("log_verbosity") == POLLER_VERBOSITY_DEBUG)) {
+			if ((substr_count($cnn_id->ErrorMsg(), "Deadlock")) || ($cnn_id->ErrorNo() == 1213) || ($cnn_id->ErrorNo() == 1205)) {
+				$errors++;
+				if ($errors > 30) {
+					cacti_log("ERROR: Too many Lock/Deadlock errors occurred! SQL:'" . str_replace("\n", "", str_replace("\r", "", str_replace("\t", " ", $sql))) ."'", TRUE);
+					return(0);
+				}else{
+					usleep(500000);
+					continue;
+				}
+			}else{
+				cacti_log("ERROR: A DB Exec Failed!, Error:'" . $cnn_id->ErrorNo() . "', SQL:\"" . str_replace("\n", "", str_replace("\r", "", str_replace("\t", " ", $sql))) . "'", FALSE);
+				return(0);
+			}
+		}
 	}
 }
 
 /* db_fetch_cell - run a 'select' sql query and return the first column of the
      first row found
    @arg $sql - the sql query to execute
+   @arg $log - whether to log error messages, defaults to true
    @arg $col_name - use this column name instead of the first one
    @returns - (bool) the output of the sql query as a single variable */
-function db_fetch_cell($sql,$col_name = '') {
+function db_fetch_cell($sql,$col_name = '', $log = TRUE) {
 	global $cnn_id;
 
-	if (!$cnn_id) { db_connect(); }
+	$sql = str_replace("  ", " ", str_replace("\n", "", str_replace("\r", "", str_replace("\t", " ", $sql))));
+
+	if (read_config_option("log_verbosity") == POLLER_VERBOSITY_DEBUG) {
+		cacti_log("DEBUG: SQL Cell: \"" . $sql . "\"", FALSE);
+	}
 
 	if ($col_name != '') {
 		$cnn_id->SetFetchMode(ADODB_FETCH_ASSOC);
@@ -81,53 +122,80 @@ function db_fetch_cell($sql,$col_name = '') {
 
 	$query = $cnn_id->Execute($sql);
 
-	if ($query) {
+	if (($query) || ($cnn_id->ErrorNo() == 1032)) {
 		if (!$query->EOF) {
 			if ($col_name != '') {
-				return($query->fields[$col_name]);
+				$column = $query->fields[$col_name];
 			}else{
-				return($query->fields[0]);
+				$column = $query->fields[0];
 			}
+
+			$query->close();
+
+			return($column);
 		}
+	}else if (($log) || (read_config_option("log_verbosity") == POLLER_VERBOSITY_DEBUG)) {
+		cacti_log("ERROR: SQL Cell Failed!, Error:'" . $cnn_id->ErrorNo() . "', SQL:\"" . str_replace("\n", "", str_replace("\r", "", str_replace("\t", " ", $sql))) . "\"", FALSE);
 	}
 }
 
 /* db_fetch_row - run a 'select' sql query and return the first row found
    @arg $sql - the sql query to execute
+   @arg $log - whether to log error messages, defaults to true
    @returns - the first row of the result as a hash */
-function db_fetch_row($sql) {
+function db_fetch_row($sql, $log = TRUE) {
 	global $cnn_id;
 
-	if (!$cnn_id) { db_connect(); }
+	$sql = str_replace("  ", " ", str_replace("\n", "", str_replace("\r", "", str_replace("\t", " ", $sql))));
+
+	if (($log) && (read_config_option("log_verbosity") == POLLER_VERBOSITY_DEBUG)) {
+		cacti_log("DEBUG: SQL Row: \"" . $sql . "\"\n", FALSE);
+	}
 
 	$cnn_id->SetFetchMode(ADODB_FETCH_ASSOC);
 	$query = $cnn_id->Execute($sql);
 
-	if ($query) {
+	if (($query) || ($cnn_id->ErrorNo() == 1032)) {
 		if (!$query->EOF) {
-			return($query->fields);
+			$fields = $query->fields;
+
+			$query->close();
+
+			return($fields);
 		}
+	}else if (($log) || (read_config_option("log_verbosity") == POLLER_VERBOSITY_DEBUG)) {
+		cacti_log("ERROR: SQL Row Failed!, Error:'" . $cnn_id->ErrorNo() . "', SQL:\"" . str_replace("\n", "", str_replace("\r", "", str_replace("\t", " ", $sql))) . "\"", FALSE);
 	}
 }
 
 /* db_fetch_assoc - run a 'select' sql query and return all rows found
    @arg $sql - the sql query to execute
+   @arg $log - whether to log error messages, defaults to true
    @returns - the entire result set as a multi-dimensional hash */
-function db_fetch_assoc($sql) {
+function db_fetch_assoc($sql, $log = TRUE) {
 	global $cnn_id;
 
-	if (!$cnn_id) { db_connect(); }
+	$sql = str_replace("  ", " ", str_replace("\n", "", str_replace("\r", "", str_replace("\t", " ", $sql))));
+
+	if (read_config_option("log_verbosity") == POLLER_VERBOSITY_DEBUG) {
+		cacti_log("DEBUG: SQL Assoc: \"" . $sql . "\"", FALSE);
+	}
 
 	$data = array();
 	$cnn_id->SetFetchMode(ADODB_FETCH_ASSOC);
 	$query = $cnn_id->Execute($sql);
 
-	if ($query) {
+	if (($query) || ($cnn_id->ErrorNo() == 1032)) {
 		while ((!$query->EOF) && ($query)) {
 			$data{sizeof($data)} = $query->fields;
 			$query->MoveNext();
 		}
+
+		$query->close();
+
 		return($data);
+	}else if (($log) || (read_config_option("log_verbosity") == POLLER_VERBOSITY_DEBUG)) {
+		cacti_log("ERROR: SQL Assoc Failed!, Error:'" . $cnn_id->ErrorNo() . "', SQL:\"" . str_replace("\n", "", str_replace("\r", "", str_replace("\t", " ", $sql))) . "\"");
 	}
 }
 
@@ -187,31 +255,41 @@ function db_replace($table_name, $array_items, $keyCols) {
    @arg $table_name - the name of the table to make the replacement in
    @arg $key_cols - the primary key(s)
    @returns - the auto incriment id column (if applicable) */
-function sql_save($array_items, $table_name, $key_cols='id') {
+function sql_save($array_items, $table_name, $key_cols = "id", $autoinc = true) {
 	global $cnn_id;
 
 	while (list ($key, $value) = each ($array_items)) {
-		if (preg_match("/^((password|md5|now)\(.*\)|null)$/i", $value)) {
-			$quote = "";
-		}else{
-			$quote = "\"";
-		}
-
-		$array_items[$key] = "$quote$value$quote";
+		$array_items[$key] = "\"" . sql_sanitize($value) . "\"";
 	}
 
-	if (!$cnn_id->Replace($table_name, $array_items, $key_cols, false)) { return 0; }
+	$replace_result = $cnn_id->Replace($table_name, $array_items, $key_cols, FALSE, $autoinc);
+
+	if ($replace_result == 0) {
+		return 0;
+	}
 
 	/* get the last AUTO_ID and return it */
-	if ($cnn_id->Insert_ID() == "0") {
-		if (isset($array_items["id"])) {
-			return str_replace("\"", "", $array_items["id"]);
-		}else{
-			return 0;
+	if (($cnn_id->Insert_ID() == "0") || ($replace_result == 1)) {
+		if (!is_array($key_cols)) {
+			if (isset($array_items[$key_cols])) {
+				return str_replace("\"", "", $array_items[$key_cols]);
+			}
 		}
+
+		return 0;
 	}else{
 		return $cnn_id->Insert_ID();
 	}
 }
 
+/* sql_sanitize - removes and quotes unwanted chars in values passed for use in SQL statements
+   @arg $value - value to sanitize
+   @return - fixed value */
+function sql_sanitize($value) {
+	//$value = str_replace("'", "''", $value);
+	$value = str_replace(";", "\;", $value);
+
+	return $value;
+}
+
 ?>
