@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004 Ian Berry                                            |
+ | Copyright (C) 2004-2008 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -13,27 +13,27 @@
  | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
  | GNU General Public License for more details.                            |
  +-------------------------------------------------------------------------+
- | cacti: a php-based graphing solution                                    |
+ | Cacti: The Complete RRDTool-based Graphing Solution                     |
  +-------------------------------------------------------------------------+
- | Most of this code has been designed, written and is maintained by       |
- | Ian Berry. See about.php for specific developer credit. Any questions   |
- | or comments regarding this code should be directed to:                  |
- | - iberry@raxnet.net                                                     |
+ | This code is designed, written, and maintained by the Cacti Group. See  |
+ | about.php and/or the AUTHORS file for specific developer information.   |
  +-------------------------------------------------------------------------+
- | - raXnet - http://www.raxnet.net/                                       |
+ | http://www.cacti.net/                                                   |
  +-------------------------------------------------------------------------+
 */
 
-function &import_xml_data(&$xml_data) {
+function &import_xml_data(&$xml_data, $import_custom_rra_settings) {
 	global $config, $hash_type_codes, $hash_version_codes;
 
 	include_once($config["library_path"] . "/xml.php");
 
+	$info_array = array();
+
 	$xml_array = xml2array($xml_data);
 
 	if (sizeof($xml_array) == 0) {
 		raise_message(7); /* xml parse error */
-		return;
+		return $info_array;
 	}
 
 	while (list($hash, $hash_array) = each($xml_array)) {
@@ -41,7 +41,7 @@ function &import_xml_data(&$xml_data) {
 		$parsed_hash = parse_xml_hash($hash);
 
 		/* invalid/wrong hash */
-		if ($parsed_hash == false) { return ; }
+		if ($parsed_hash == false) { return $info_array; }
 
 		if (isset($dep_hash_cache{$parsed_hash["type"]})) {
 			array_push($dep_hash_cache{$parsed_hash["type"]}, $parsed_hash);
@@ -68,7 +68,7 @@ function &import_xml_data(&$xml_data) {
 					$hash_cache += xml_to_graph_template($dep_hash_cache[$type][$i]["hash"], $hash_array, $hash_cache);
 					break;
 				case 'data_template':
-					$hash_cache += xml_to_data_template($dep_hash_cache[$type][$i]["hash"], $hash_array, $hash_cache);
+					$hash_cache += xml_to_data_template($dep_hash_cache[$type][$i]["hash"], $hash_array, $hash_cache, $import_custom_rra_settings);
 					break;
 				case 'host_template':
 					$hash_cache += xml_to_host_template($dep_hash_cache[$type][$i]["hash"], $hash_array, $hash_cache);
@@ -86,11 +86,15 @@ function &import_xml_data(&$xml_data) {
 					$hash_cache += xml_to_cdef($dep_hash_cache[$type][$i]["hash"], $hash_array, $hash_cache);
 					break;
 				case 'round_robin_archive':
-					$hash_cache += xml_to_round_robin_archive($dep_hash_cache[$type][$i]["hash"], $hash_array, $hash_cache);
+					if ($import_custom_rra_settings === true) {
+						$hash_cache += xml_to_round_robin_archive($dep_hash_cache[$type][$i]["hash"], $hash_array, $hash_cache);
+					}
 					break;
 				}
 
-				$info_array[$type]{isset($info_array[$type]) ? count($info_array[$type]) : 0} = $_SESSION["import_debug_info"];
+				if (isset($_SESSION["import_debug_info"])) {
+					$info_array[$type]{isset($info_array[$type]) ? count($info_array[$type]) : 0} = $_SESSION["import_debug_info"];
+				}
 
 				kill_session_var("import_debug_info");
 			}
@@ -114,7 +118,7 @@ function &xml_to_graph_template($hash, &$xml_array, &$hash_cache) {
 
 	/* import into: graph_templates_graph */
 	unset($save);
-	$save["id"] = (empty($_graph_template_id) ? "0" : db_fetch_cell("select graph_templates_graph.id from graph_templates,graph_templates_graph where graph_templates.id=graph_templates_graph.graph_template_id and graph_templates.id=$graph_template_id and graph_templates_graph.local_graph_id=0"));
+	$save["id"] = (empty($_graph_template_id) ? "0" : db_fetch_cell("select graph_templates_graph.id from (graph_templates,graph_templates_graph) where graph_templates.id=graph_templates_graph.graph_template_id and graph_templates.id=$graph_template_id and graph_templates_graph.local_graph_id=0"));
 	$save["graph_template_id"] = $graph_template_id;
 
 	/* parse information from the hash */
@@ -132,7 +136,7 @@ function &xml_to_graph_template($hash, &$xml_array, &$hash_cache) {
 			if (($field_name == "unit_exponent_value") && (get_version_index($parsed_hash["version"]) < get_version_index("0.8.5")) && ($xml_array["graph"][$field_name] == "0")) { /* backwards compatability */
 				$save[$field_name] = "";
 			}else{
-				$save[$field_name] = addslashes($xml_array["graph"][$field_name]);
+				$save[$field_name] = addslashes(xml_character_decode($xml_array["graph"][$field_name]));
 			}
 		}
 	}
@@ -171,7 +175,7 @@ function &xml_to_graph_template($hash, &$xml_array, &$hash_cache) {
 
 						$save[$field_name] = $color_id;
 					}else{
-						$save[$field_name] = addslashes($item_array[$field_name]);
+						$save[$field_name] = addslashes(xml_character_decode($item_array[$field_name]));
 					}
 				}
 			}
@@ -201,7 +205,7 @@ function &xml_to_graph_template($hash, &$xml_array, &$hash_cache) {
 			while (list($field_name, $field_array) = each($fields_graph_template_input_edit)) {
 				/* make sure this field exists in the xml array first */
 				if (isset($item_array[$field_name])) {
-					$save[$field_name] = addslashes($item_array[$field_name]);
+					$save[$field_name] = addslashes(xml_character_decode($item_array[$field_name]));
 				}
 			}
 
@@ -236,7 +240,7 @@ function &xml_to_graph_template($hash, &$xml_array, &$hash_cache) {
 	return $hash_cache;
 }
 
-function &xml_to_data_template($hash, &$xml_array, &$hash_cache) {
+function &xml_to_data_template($hash, &$xml_array, &$hash_cache, $import_custom_rra_settings) {
 	global $struct_data_source, $struct_data_source_item;
 
 	/* import into: data_template */
@@ -251,7 +255,7 @@ function &xml_to_data_template($hash, &$xml_array, &$hash_cache) {
 
 	/* import into: data_template_data */
 	unset($save);
-	$save["id"] = (empty($_data_template_id) ? "0" : db_fetch_cell("select data_template_data.id from data_template,data_template_data where data_template.id=data_template_data.data_template_id and data_template.id=$data_template_id and data_template_data.local_data_id=0"));
+	$save["id"] = (empty($_data_template_id) ? "0" : db_fetch_cell("select data_template_data.id from (data_template,data_template_data) where data_template.id=data_template_data.data_template_id and data_template.id=$data_template_id and data_template_data.local_data_id=0"));
 	$save["data_template_id"] = $data_template_id;
 
 	reset($struct_data_source);
@@ -267,26 +271,43 @@ function &xml_to_data_template($hash, &$xml_array, &$hash_cache) {
 			if (ereg("hash_([a-f0-9]{2})([a-f0-9]{4})([a-f0-9]{32})", $xml_array["ds"][$field_name])) {
 				$save[$field_name] = resolve_hash_to_id($xml_array["ds"][$field_name], $hash_cache);
 			}else{
-				$save[$field_name] = addslashes($xml_array["ds"][$field_name]);
+				$save[$field_name] = addslashes(xml_character_decode($xml_array["ds"][$field_name]));
 			}
 		}
 	}
 
+	/* use the polling interval as the step if we are to use the default rra settings */
+	if ($import_custom_rra_settings === false) {
+		$save["rrd_step"] = read_config_option("poller_interval");
+	}
+
 	$data_template_data_id = sql_save($save, "data_template_data");
 
-	/* import into: data_template_data_rra */
-	$hash_items = explode("|", $xml_array["ds"]["rra_items"]);
+	/* use custom rra settings from the xml */
+	if ($import_custom_rra_settings === true) {
+		/* import into: data_template_data_rra */
+		$hash_items = explode("|", $xml_array["ds"]["rra_items"]);
 
-	if (!empty($hash_items[0])) {
-		for ($i=0; $i<count($hash_items); $i++) {
-			/* parse information from the hash */
-			$parsed_hash = parse_xml_hash($hash_items[$i]);
+		if (!empty($hash_items[0])) {
+			for ($i=0; $i<count($hash_items); $i++) {
+				/* parse information from the hash */
+				$parsed_hash = parse_xml_hash($hash_items[$i]);
 
-			/* invalid/wrong hash */
-			if ($parsed_hash == false) { return false; }
+				/* invalid/wrong hash */
+				if ($parsed_hash == false) { return false; }
 
-			if (isset($hash_cache["round_robin_archive"]{$parsed_hash["hash"]})) {
-				db_execute("replace into data_template_data_rra (data_template_data_id,rra_id) values ($data_template_data_id," . $hash_cache["round_robin_archive"]{$parsed_hash["hash"]} . ")");
+				if (isset($hash_cache["round_robin_archive"]{$parsed_hash["hash"]})) {
+					db_execute("replace into data_template_data_rra (data_template_data_id,rra_id) values ($data_template_data_id," . $hash_cache["round_robin_archive"]{$parsed_hash["hash"]} . ")");
+				}
+			}
+		}
+	/* use all rras by default */
+	}else{
+		$rras = db_fetch_assoc("select id from rra");
+
+		if (is_array($rras)) {
+			foreach ($rras as $rra) {
+				db_execute("replace into data_template_data_rra (data_template_data_id,rra_id) values ($data_template_data_id," . $rra["id"] . ")");
 			}
 		}
 	}
@@ -319,11 +340,16 @@ function &xml_to_data_template($hash, &$xml_array, &$hash_cache) {
 					if (ereg("hash_([a-f0-9]{2})([a-f0-9]{4})([a-f0-9]{32})", $item_array[$field_name])) {
 						$save[$field_name] = resolve_hash_to_id($item_array[$field_name], $hash_cache);
 					}else{
-						$save[$field_name] = addslashes($item_array[$field_name]);
+						$save[$field_name] = addslashes(xml_character_decode($item_array[$field_name]));
 					}
 				}
 			}
 
+			/* use the polling interval * 2 as the heartbeat if we are to use the default rra settings */
+			if ($import_custom_rra_settings === false) {
+				$save["rrd_heartbeat"] = read_config_option("poller_interval") * 2;
+			}
+
 			$data_template_rrd_id = sql_save($save, "data_template_rrd");
 
 			$hash_cache["data_template_item"]{$parsed_hash["hash"]} = $data_template_rrd_id;
@@ -337,9 +363,9 @@ function &xml_to_data_template($hash, &$xml_array, &$hash_cache) {
 			$save["data_template_data_id"] = $data_template_data_id;
 			$save["data_input_field_id"] = resolve_hash_to_id($item_array["data_input_field_id"], $hash_cache);
 			$save["t_value"] = $item_array["t_value"];
-			$save["value"] = $item_array["value"];
+			$save["value"] = addslashes(xml_character_decode($item_array["value"]));
 
-			sql_save($save, "data_input_data", array("data_template_data_id", "data_input_field_id"));
+			sql_save($save, "data_input_data", array("data_template_data_id", "data_input_field_id"), false);
 		}
 	}
 
@@ -399,7 +425,7 @@ function &xml_to_data_query($hash, &$xml_array, &$hash_cache) {
 					if (ereg("hash_([a-f0-9]{2})([a-f0-9]{4})([a-f0-9]{32})", $item_array[$field_name])) {
 						$save[$field_name] = resolve_hash_to_id($item_array[$field_name], $hash_cache);
 					}else{
-						$save[$field_name] = addslashes($item_array[$field_name]);
+						$save[$field_name] = addslashes(xml_character_decode($item_array[$field_name]));
 					}
 				}
 			}
@@ -417,7 +443,7 @@ function &xml_to_data_query($hash, &$xml_array, &$hash_cache) {
 					$save["data_template_rrd_id"] = resolve_hash_to_id($sub_item_array["data_template_rrd_id"], $hash_cache);
 					$save["snmp_field_name"] = $sub_item_array["snmp_field_name"];
 
-					sql_save($save, "snmp_query_graph_rrd", array("snmp_query_graph_id", "data_template_id", "data_template_rrd_id"));
+					sql_save($save, "snmp_query_graph_rrd", array("snmp_query_graph_id", "data_template_id", "data_template_rrd_id"), false);
 				}
 			}
 
@@ -437,7 +463,7 @@ function &xml_to_data_query($hash, &$xml_array, &$hash_cache) {
 					$save["snmp_query_graph_id"] = $data_query_graph_id;
 					$save["sequence"] = $sub_item_array["sequence"];
 					$save["field_name"] = $sub_item_array["field_name"];
-					$save["text"] = $sub_item_array["text"];
+					$save["text"] = xml_character_decode($sub_item_array["text"]);
 
 					$data_query_graph_sv_id = sql_save($save, "snmp_query_graph_sv");
 
@@ -462,7 +488,7 @@ function &xml_to_data_query($hash, &$xml_array, &$hash_cache) {
 					$save["data_template_id"] = resolve_hash_to_id($sub_item_array["data_template_id"], $hash_cache);
 					$save["sequence"] = $sub_item_array["sequence"];
 					$save["field_name"] = $sub_item_array["field_name"];
-					$save["text"] = $sub_item_array["text"];
+					$save["text"] = xml_character_decode($sub_item_array["text"]);
 
 					$data_query_graph_rrd_sv_id = sql_save($save, "snmp_query_graph_rrd_sv");
 
@@ -492,7 +518,7 @@ function &xml_to_gprint_preset($hash, &$xml_array, &$hash_cache) {
 	while (list($field_name, $field_array) = each($fields_grprint_presets_edit)) {
 		/* make sure this field exists in the xml array first */
 		if (isset($xml_array[$field_name])) {
-			$save[$field_name] = addslashes($xml_array[$field_name]);
+			$save[$field_name] = addslashes(xml_character_decode($xml_array[$field_name]));
 		}
 	}
 
@@ -520,7 +546,7 @@ function &xml_to_round_robin_archive($hash, &$xml_array, &$hash_cache) {
 	while (list($field_name, $field_array) = each($fields_rra_edit)) {
 		/* make sure this field exists in the xml array first */
 		if (isset($xml_array[$field_name])) {
-			$save[$field_name] = addslashes($xml_array[$field_name]);
+			$save[$field_name] = addslashes(xml_character_decode($xml_array[$field_name]));
 		}
 	}
 
@@ -557,7 +583,7 @@ function &xml_to_host_template($hash, &$xml_array, &$hash_cache) {
 	while (list($field_name, $field_array) = each($fields_host_template_edit)) {
 		/* make sure this field exists in the xml array first */
 		if (isset($xml_array[$field_name])) {
-			$save[$field_name] = addslashes($xml_array[$field_name]);
+			$save[$field_name] = addslashes(xml_character_decode($xml_array[$field_name]));
 		}
 	}
 
@@ -625,7 +651,7 @@ function &xml_to_cdef($hash, &$xml_array, &$hash_cache) {
 	while (list($field_name, $field_array) = each($fields_cdef_edit)) {
 		/* make sure this field exists in the xml array first */
 		if (isset($xml_array[$field_name])) {
-			$save[$field_name] = addslashes($xml_array[$field_name]);
+			$save[$field_name] = addslashes(xml_character_decode($xml_array[$field_name]));
 		}
 	}
 
@@ -652,7 +678,7 @@ function &xml_to_cdef($hash, &$xml_array, &$hash_cache) {
 			while (list($field_name, $field_array) = each($fields_cdef_item_edit)) {
 				/* make sure this field exists in the xml array first */
 				if (isset($item_array[$field_name])) {
-					$save[$field_name] = addslashes($item_array[$field_name]);
+					$save[$field_name] = addslashes(xml_character_decode($item_array[$field_name]));
 				}
 			}
 
@@ -685,6 +711,11 @@ function &xml_to_data_input_method($hash, &$xml_array, &$hash_cache) {
 	while (list($field_name, $field_array) = each($fields_data_input_edit)) {
 		/* make sure this field exists in the xml array first */
 		if (isset($xml_array[$field_name])) {
+			/* fix issue with data input method importing and white spaces */
+			if ($field_name == "input_string") {
+				$xml_array[$field_name] = str_replace("><", "> <", $xml_array[$field_name]);
+			}
+
 			$save[$field_name] = addslashes(xml_character_decode($xml_array[$field_name]));
 		}
 	}
@@ -712,7 +743,7 @@ function &xml_to_data_input_method($hash, &$xml_array, &$hash_cache) {
 			while (list($field_name, $field_array) = each($fields_data_input_field_edit)) {
 				/* make sure this field exists in the xml array first */
 				if (isset($item_array[$field_name])) {
-					$save[$field_name] = addslashes($item_array[$field_name]);
+					$save[$field_name] = addslashes(xml_character_decode($item_array[$field_name]));
 				}
 			}
 
@@ -724,7 +755,7 @@ function &xml_to_data_input_method($hash, &$xml_array, &$hash_cache) {
 
 	/* update field use counter cache if possible */
 	if ((isset($xml_array["input_string"])) && (!empty($data_input_id))) {
-		generate_data_input_field_sequences($xml_array["input_string"], $data_input_id, "in");
+		generate_data_input_field_sequences($xml_array["input_string"], $data_input_id);
 	}
 
 	/* status information that will be presented to the user */
@@ -821,7 +852,7 @@ function check_hash_type($hash_type) {
 	}
 
 	if (!isset($current_type)) {
-		// error: cannot find type
+		raise_message(18); /* error: cannot find type */
 		return false;
 	}
 
@@ -848,13 +879,13 @@ function check_hash_version($hash_version) {
 	}
 
 	if (!isset($current_version_index)) {
-		// error: current cacti version does not exist!
+		raise_message(15); /* error: current cacti version does not exist! */
 		return false;
 	}elseif (!isset($hash_version_index)) {
-		// error: hash version does not exist!
+		raise_message(16); /* error: hash version does not exist! */
 		return false;
 	}elseif ($hash_version_index > $current_version_index) {
-		// error: hash made with a newer version of cacti
+		raise_message(17); /* error: hash made with a newer version of cacti */
 		return false;
 	}
 
