@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004 Ian Berry                                            |
+ | Copyright (C) 2004-2008 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -13,14 +13,12 @@
  | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
  | GNU General Public License for more details.                            |
  +-------------------------------------------------------------------------+
- | cacti: a php-based graphing solution                                    |
+ | Cacti: The Complete RRDTool-based Graphing Solution                     |
  +-------------------------------------------------------------------------+
- | Most of this code has been designed, written and is maintained by       |
- | Ian Berry. See about.php for specific developer credit. Any questions   |
- | or comments regarding this code should be directed to:                  |
- | - iberry@raxnet.net                                                     |
+ | This code is designed, written, and maintained by the Cacti Group. See  |
+ | about.php and/or the AUTHORS file for specific developer information.   |
  +-------------------------------------------------------------------------+
- | - raXnet - http://www.raxnet.net/                                       |
+ | http://www.cacti.net/                                                   |
  +-------------------------------------------------------------------------+
 */
 
@@ -36,6 +34,16 @@ function repopulate_poller_cache() {
 	}
 }
 
+function update_poller_cache_from_query($host_id, $data_query_id) {
+	$poller_data = db_fetch_assoc("select id from data_local where host_id = '$host_id' and snmp_query_id = '$data_query_id'");
+
+	if (sizeof($poller_data) > 0) {
+		foreach ($poller_data as $data) {
+			update_poller_cache($data["id"]);
+		}
+	}
+}
+
 function update_poller_cache($local_data_id, $truncate_performed = false) {
 	global $config;
 
@@ -47,8 +55,9 @@ function update_poller_cache($local_data_id, $truncate_performed = false) {
 		data_input.type_id,
 		data_template_data.id as data_template_data_id,
 		data_template_data.data_template_id,
-		data_template_data.active
-		from data_template_data,data_input
+		data_template_data.active,
+		data_template_data.rrd_step
+		from (data_template_data,data_input)
 		where data_template_data.data_input_id=data_input.id
 		and data_template_data.local_data_id=$local_data_id");
 
@@ -61,20 +70,25 @@ function update_poller_cache($local_data_id, $truncate_performed = false) {
 
 	/* we have to perform some additional sql queries if this is a "query" */
 	if (($data_input["type_id"] == DATA_INPUT_TYPE_SNMP_QUERY) ||
-		 ($data_input["type_id"] == DATA_INPUT_TYPE_SCRIPT_QUERY) ||
-		 ($data_input["type_id"] == DATA_INPUT_TYPE_PHP_SCRIPT_SERVER) ||
-		 ($data_input["type_id"] == DATA_INPUT_TYPE_QUERY_SCRIPT_SERVER)){
+		($data_input["type_id"] == DATA_INPUT_TYPE_SCRIPT_QUERY) ||
+		($data_input["type_id"] == DATA_INPUT_TYPE_QUERY_SCRIPT_SERVER)){
 		$field = data_query_field_list($data_input["data_template_data_id"]);
 
 		if (($data_input["type_id"] != DATA_INPUT_TYPE_PHP_SCRIPT_SERVER) &&
-			(empty($field))) { return; }
+			(empty($field["output_type"]))) { return; }
+
+		if (strlen($field["output_type"])) {
+			$output_type_sql = "and snmp_query_graph_rrd.snmp_query_graph_id=" . $field["output_type"];
+		}else{
+			$output_type_sql = "";
+		}
 
 		$outputs = db_fetch_assoc("select
 			snmp_query_graph_rrd.snmp_field_name,
 			data_template_rrd.id as data_template_rrd_id
-			from snmp_query_graph_rrd,data_template_rrd
+			from (snmp_query_graph_rrd,data_template_rrd)
 			where snmp_query_graph_rrd.data_template_rrd_id=data_template_rrd.local_data_template_rrd_id
-			and snmp_query_graph_rrd.snmp_query_graph_id=" . $field["output_type"] . "
+			$output_type_sql
 			and snmp_query_graph_rrd.data_template_id=" . $data_input["data_template_id"] . "
 			and data_template_rrd.local_data_id=$local_data_id");
 	}
@@ -102,7 +116,7 @@ function update_poller_cache($local_data_id, $truncate_performed = false) {
 				$data_source_item_name = "";
 			}
 
-			api_poller_cache_item_add($data_source["host_id"], array(), $local_data_id, $action, $data_source_item_name, 1, addslashes($script_path));
+			api_poller_cache_item_add($data_source["host_id"], array(), $local_data_id, $data_input["rrd_step"], $action, $data_source_item_name, 1, addslashes($script_path));
 		}else if ($data_input["type_id"] == DATA_INPUT_TYPE_SNMP) { /* snmp */
 			$host_fields = array_rekey(db_fetch_assoc("select
 				data_input_fields.type_code,
@@ -114,6 +128,10 @@ function update_poller_cache($local_data_id, $truncate_performed = false) {
 					or data_input_fields.type_code='snmp_community'
 					or data_input_fields.type_code='snmp_username'
 					or data_input_fields.type_code='snmp_password'
+					or data_input_fields.type_code='snmp_auth_protocol'
+					or data_input_fields.type_code='snmp_priv_passphrase'
+					or data_input_fields.type_code='snmp_priv_protocol'
+					or data_input_fields.type_code='snmp_context'
 					or data_input_fields.type_code='snmp_version'
 					or data_input_fields.type_code='snmp_port'
 					or data_input_fields.type_code='snmp_timeout')
@@ -121,7 +139,7 @@ function update_poller_cache($local_data_id, $truncate_performed = false) {
 
 			$data_template_rrd_id = db_fetch_cell("select id from data_template_rrd where local_data_id=$local_data_id");
 
-			api_poller_cache_item_add($data_source["host_id"], $host_fields, $local_data_id, 0, get_data_source_item_name($data_template_rrd_id), 1, (isset($host_fields["snmp_oid"]) ? $host_fields["snmp_oid"] : ""));
+			api_poller_cache_item_add($data_source["host_id"], $host_fields, $local_data_id, $data_input["rrd_step"], 0, get_data_source_item_name($data_template_rrd_id), 1, (isset($host_fields["snmp_oid"]) ? $host_fields["snmp_oid"] : ""));
 		}else if ($data_input["type_id"] == DATA_INPUT_TYPE_SNMP_QUERY) { /* snmp query */
 			$snmp_queries = get_data_query_array($data_source["snmp_query_id"]);
 
@@ -132,7 +150,7 @@ function update_poller_cache($local_data_id, $truncate_performed = false) {
 				}
 
 				if (!empty($oid)) {
-					api_poller_cache_item_add($data_source["host_id"], array(), $local_data_id, 0, get_data_source_item_name($output["data_template_rrd_id"]), sizeof($outputs), $oid);
+					api_poller_cache_item_add($data_source["host_id"], array(), $local_data_id, $data_input["rrd_step"], 0, get_data_source_item_name($output["data_template_rrd_id"]), sizeof($outputs), $oid);
 				}
 			}
 			}
@@ -158,7 +176,7 @@ function update_poller_cache($local_data_id, $truncate_performed = false) {
 					}
 
 					if (isset($script_path)) {
-						api_poller_cache_item_add($data_source["host_id"], array(), $local_data_id, $action, get_data_source_item_name($output["data_template_rrd_id"]), sizeof($outputs), addslashes($script_path));
+						api_poller_cache_item_add($data_source["host_id"], array(), $local_data_id, $data_input["rrd_step"], $action, get_data_source_item_name($output["data_template_rrd_id"]), sizeof($outputs), addslashes($script_path));
 					}
 				}
 			}
@@ -191,7 +209,7 @@ function push_out_host($host_id, $local_data_id = 0, $data_template_id = 0) {
 		data_template_data.data_input_id,
 		data_template_data.local_data_id,
 		data_template_data.local_data_template_data_id
-		from data_local,data_template_data
+		from (data_local,data_template_data)
 		where " . (empty($local_data_id) ? "data_local.host_id=$host_id" : "data_local.id=$local_data_id") . "
 		and data_local.id=data_template_data.local_data_id
 		and data_template_data.data_input_id>0");
@@ -472,4 +490,4 @@ function duplicate_host_template($_host_template_id, $host_template_title) {
 	}
 }
 
-?>
\ No newline at end of file
+?>
